@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -371,6 +371,9 @@ namespace vapp {
               pBlock->fTextureScale = 1.0f;              
               pBlock->ID = _CreateBlockName();
               pBlock->Texture = "default";
+              
+              //float ff = 0;
+              
               for(int i=0;i<m_NewBlockX.size();i++) {
                 LevelBlockVertex *pVertex = new LevelBlockVertex;
                 pBlock->Vertices.push_back( pVertex );
@@ -383,7 +386,22 @@ namespace vapp {
                 pVertex->b = 255;
                 pVertex->a = 255;
                 pVertex->bSelected = false;
+                
+                //int inext = i+1;
+//                int iprev = i-1;
+                //if(inext >= m_NewBlockX.size()) inext = 0;
+//                if(iprev < 0) iprev = m_NewBlockX.size()-1;
+                
+                //float ffX1 = m_NewBlockX[i] - pBlock->fPosX;
+                //float ffY1 = m_NewBlockY[i] - pBlock->fPosY;
+                //float ffX2 = m_NewBlockX[inext] - pBlock->fPosX;
+                //float ffY2 = m_NewBlockY[inext] - pBlock->fPosY;
+                //
+                //ff += ffX1*ffX2 + ffY1*ffY2;
               }
+              
+              //printf("[%f]\n",ff);
+              
               setState(ES_DEFAULT);
             }
             else {
@@ -419,7 +437,7 @@ namespace vapp {
             TextureSelectionTool Sel;
             std::vector<Texture *> TexList;
             
-            /* Only list textures tagget with "BLK" */
+            /* Only list textures tagged with "BLK" */
             TexList = TexMan.fetchTaggedTextures("BLK");
             
             Vector2f A=Vector2f(17,17);
@@ -1208,7 +1226,30 @@ namespace vapp {
 					  m_Log.msg("Level parameters changed.");
   					
             m_pLevelSrc->setFileName( Entries[0].Value );
-            m_pLevelSrc->setID( Entries[1].Value );
+            
+            /* Make sure invalid chars isn't saved in ID */
+            std::string ID = Entries[1].Value;
+
+            int i=0;
+            while(1) {
+              if(i >= ID.length()) break;
+              
+              if((ID[i] >= 'a' && ID[i] <= 'z') ||
+                (ID[i] >= 'A' && ID[i] <= 'Z') ||
+                (ID[i] >= '0' && ID[i] <= '9') || ID[i]=='_') {
+                /* This is ok */
+                i++;
+              }
+              else {
+                /* Not ok */
+                ID.erase(ID.begin() + i);
+              }            
+            }
+            
+            if(ID == "") ID = "noname";
+                        
+            m_pLevelSrc->setID(ID);
+            
             m_pLevelSrc->setScriptFile( Entries[2].Value );
             m_pLevelSrc->getLevelInfo()->Name = Entries[3].Value;
             m_pLevelSrc->getLevelInfo()->Date = Entries[4].Value;
@@ -1308,7 +1349,7 @@ namespace vapp {
         m_Log.msg("Playing it.");
         
         char cBuf[256];
-        sprintf(cBuf,"%s -win -level %s",XMOTOCOMMAND,m_pLevelSrc->getID().c_str());
+        sprintf(cBuf,"%s -win -level \"%s\"",XMOTOCOMMAND,m_pLevelSrc->getID().c_str());
         system(cBuf);      
         setState(ES_DEFAULT);			          
 			  break;
