@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -320,12 +320,14 @@ namespace vapp {
     pMsgBox->makeActive();    
     if(bTextInput)
       pMsgBox->enableTextInput();
-
-    /* Disable all siblings */
-    for(int i=0;i<getChildren().size()-1;i++) {      
-      pMsgBox->getSiblingStates().push_back(!getChildren()[i]->isDisabled());
-      getChildren()[i]->enableWindow(false);     
-    }
+    
+    /* Disable all siblings (if there's not already a msg-box) */
+    //if(!m_bActiveMsgBox) {
+      for(int i=0;i<getChildren().size()-1;i++) {      
+        pMsgBox->getSiblingStates().push_back(!getChildren()[i]->isDisabled());
+        getChildren()[i]->enableWindow(false);     
+      }
+    //}
     
     /* Create text static */
     int nStaticY=0;
@@ -368,6 +370,9 @@ namespace vapp {
       pMsgBox->addButton(pButton);
       nCX+=115;
     }
+    
+    /* Set msgbox flag */
+    m_bActiveMsgBox = true;
 
     return pMsgBox;                                
   }
@@ -631,17 +636,46 @@ FRAME_BR (187,198) (8x8)
   void UIRoot::paint(void) {
     UIRect Screen;
     
+    /* Clip to full screen */
     Screen.nX = 0;
     Screen.nY = 0;
     Screen.nWidth = getApp()->getDispWidth();
     Screen.nHeight = getApp()->getDispHeight();
       
+    /* Draw root's children */
     glEnable(GL_SCISSOR_TEST);
 
     for(int i=0;i<getChildren().size();i++)
       _RootPaint(0,0,getChildren()[i],&Screen);
 
     glDisable(GL_SCISSOR_TEST);
+
+    /* Context help? */
+    if(m_bShowContextMenu) {
+      int nContextHelpHeight = 20;
+      
+      /* Shade out bottom of screen */
+      glEnable(GL_BLEND);
+      glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
+      glBegin(GL_POLYGON);
+      glColor4f(0,0,0,0);
+      getApp()->glVertex(0,getApp()->getDispHeight()-nContextHelpHeight);
+      getApp()->glVertex(getApp()->getDispWidth(),getApp()->getDispHeight()-nContextHelpHeight);
+      glColor4f(0,0,0,0.7);
+      getApp()->glVertex(getApp()->getDispWidth(),getApp()->getDispHeight());
+      getApp()->glVertex(0,getApp()->getDispHeight());
+      glEnd();
+      glDisable(GL_BLEND);
+        
+      if(!m_CurrentContextHelp.empty()) {
+        /* Print help string */
+        int nX1,nY1,nX2,nY2;
+        getTextExt(m_CurrentContextHelp,&nX1,&nY1,&nX2,&nY2);
+        
+        UITextDraw::printRaw(getFont(),getApp()->getDispWidth()-(nX2-nY1),getApp()->getDispHeight()-5,
+                            m_CurrentContextHelp,MAKE_COLOR(255,255,0,255));
+      }
+    }
   }
   
   void UIRoot::mouseLDown(int x,int y) {
@@ -759,7 +793,20 @@ FRAME_BR (187,198) (8x8)
           case UI_ROOT_MOUSE_RBUTTON_DOWN: pWindow->mouseRDown(wx,wy); break;
           case UI_ROOT_MOUSE_LBUTTON_UP: pWindow->mouseLUp(wx,wy); break;
           case UI_ROOT_MOUSE_RBUTTON_UP: pWindow->mouseRUp(wx,wy); break;
-          case UI_ROOT_MOUSE_HOVER: pWindow->mouseHover(wx,wy); break;
+          case UI_ROOT_MOUSE_HOVER: {              
+              /* Post mouse-hover event to window */
+              pWindow->mouseHover(wx,wy); 
+              
+              /* By the way, does it want to offer context-help? (only 
+                 if mouse have actually moved) */
+              if(getApp()->haveMouseMoved()) {
+                std::string SubCHelp = pWindow->subContextHelp(wx,wy);
+                if(SubCHelp != "") m_CurrentContextHelp = SubCHelp;
+                else m_CurrentContextHelp = pWindow->getContextHelp();
+              }
+              
+              break;
+            }
           case UI_ROOT_MOUSE_WHEEL_UP: pWindow->mouseWheelUp(wx,wy); break;
           case UI_ROOT_MOUSE_WHEEL_DOWN: pWindow->mouseWheelDown(wx,wy); break;
           case UI_ROOT_MOUSE_DOUBLE_CLICK: pWindow->mouseLDoubleClick(wx,wy); break;
@@ -988,7 +1035,7 @@ FRAME_BR (187,198) (8x8)
     glDisable(GL_BLEND);
     glDisable(GL_TEXTURE_2D);
   }
-  
+    
   void UITextDraw::getTextExt(UIFont *pFont,std::string Text,int *pnMinX,int *pnMinY,int *pnMaxX,int *pnMaxY) {
     int cx=0,cy=0;
     
@@ -1003,10 +1050,14 @@ FRAME_BR (187,198) (8x8)
         cy += (pFont->Chars['|'].nHeight*3)/2;
         cx = 0;
       }
+      else if(nChar == ' ') {
+        cx += pFont->Chars['-'].nIncX;
+        if(pnMaxX) *pnMaxX += pFont->Chars['-'].nIncX;
+      }
       else if(pFont->Chars[nChar].bAvail) {
         int x1 = cx + pFont->Chars[nChar].nOffsetX;
         int y1 = cy - pFont->Chars[nChar].nOffsetY;
-        int x2 = x1 + pFont->Chars[nChar].nWidth;
+        int x2 = cx + pFont->Chars[nChar].nIncX;
         int y2 = y1 + pFont->Chars[nChar].nHeight;
 
         if(pnMinX && x1<*pnMinX) *pnMinX = x1;
@@ -1016,8 +1067,10 @@ FRAME_BR (187,198) (8x8)
         
         cx += pFont->Chars[nChar].nIncX;
       }
-      else
+      else {
         cx += pFont->Chars['-'].nIncX;
+        if(pnMaxX) *pnMaxX += pFont->Chars['-'].nIncX;
+      }
     }  
   }
   
@@ -1166,13 +1219,19 @@ FRAME_BR (187,198) (8x8)
         if(nBest>=0) {
           deactivate(this);
           Map[nBest].pWindow->setActive(true);          
+          
+          m_CurrentContextHelp = Map[nBest].pWindow->getContextHelp();
         }
         else {
           deactivate(this);
-          if(dx<0 || dy<0)
+          if(dx<0 || dy<0) {
             Map[0].pWindow->setActive(true);
-          else if(dx>0 || dy>0)
+            m_CurrentContextHelp = Map[0].pWindow->getContextHelp();
+          }
+          else if(dx>0 || dy>0) {
             Map[nNum-1].pWindow->setActive(true);
+            m_CurrentContextHelp = Map[nNum-1].pWindow->getContextHelp();
+          }
         }
       }
     }
