@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -194,6 +194,7 @@ namespace vapp {
         m_bDisable = false;
         m_bActive = false;
         m_nGroup = 0;
+        m_bActiveMsgBox = false;
         setPrimaryChild(NULL);
         setOpacity(100);
       }
@@ -220,6 +221,7 @@ namespace vapp {
       virtual bool keyDown(int nKey,int nChar) {return false;}
       virtual bool keyUp(int nKey) {return false;}
       virtual bool offerActivation(void) {return false;}
+      virtual std::string subContextHelp(int x,int y) {return "";}
 
       /* Painting */            
       void putText(int x,int y,std::string Text);      
@@ -278,6 +280,9 @@ namespace vapp {
       void setActive(bool b) {m_bActive = b;}
       int getGroup(void) {return m_nGroup;}
       void setGroup(int n) {m_nGroup = n;}
+      void setContextHelp(const std::string &s) {m_ContextHelp = s;}
+      const std::string &getContextHelp(void) {return m_ContextHelp;}
+      void clearMsgBoxActive(void) {m_bActiveMsgBox = false; /* pain */}
     
     protected:
       /* Protected interface */
@@ -289,6 +294,7 @@ namespace vapp {
     
     private:
       /* Data */
+      std::string m_ContextHelp;                /* Context help */
       UIWindow *m_pPrimaryChild;                /* If !=null, this child is activated when
                                                    the window transists from "hidden" to "shown". 
                                                    useful for specifying "default" buttons in menus */
@@ -307,8 +313,31 @@ namespace vapp {
       bool m_bDisable;                          /* Disable window */
       bool m_bActive;                           /* Only one window should be active at a time */
       int m_nGroup;                             /* In window group */    
+      bool m_bActiveMsgBox;                     /* Message box active */
   };  
 
+	/*===========================================================================
+	UI edit
+  ===========================================================================*/
+  class UIEdit : public UIWindow {
+    public:
+      UIEdit() {}
+      UIEdit(UIWindow *pParent,int x=0,int y=0,std::string Caption="",int nWidth=0,int nHeight=0) {
+        initW(pParent,x,y,Caption,nWidth,nHeight);
+        m_nCursorPos = 0;
+      }      
+    
+      /* Methods */
+      virtual void paint(void);
+      virtual bool keyDown(int nKey,int nChar);
+      
+      virtual bool offerActivation(void) {return true;}
+
+    private:
+      /* Data */
+      int m_nCursorPos;
+    
+  };
 
 	/*===========================================================================
 	UI frame
@@ -458,6 +487,8 @@ namespace vapp {
         m_bBackgroundShade = false;
         
         m_pDarkBlobTexture = UITexture::load("./Textures/UI/DarkBlob.png");                         
+        
+        m_pCustomBackgroundTexture = NULL;
       }      
 
       /* Methods */
@@ -469,6 +500,7 @@ namespace vapp {
       UIAlign getVAlign(void) {return m_VAlign;}
       UIAlign getHAlign(void) {return m_HAlign;}
       void setBackgroundShade(bool b) {m_bBackgroundShade=b;}
+      void setBackground(Texture *p) {m_pCustomBackgroundTexture = p;}
       
     private:
       /* Data */
@@ -476,6 +508,7 @@ namespace vapp {
       bool m_bBackgroundShade;
       
       Texture *m_pDarkBlobTexture;
+      Texture *m_pCustomBackgroundTexture;
   };
   
 	/*===========================================================================
@@ -555,12 +588,16 @@ namespace vapp {
         m_nScroll = 0;
         m_nSelected = 0;
         m_pEnterButton = NULL;
-        
+        m_bSort = false;
         m_bItemActivated = false;
         
         m_bScrollDownPressed = m_bScrollUpPressed = false;
         
         m_bScrollDownHover = m_bScrollUpHover = false;
+        
+	m_bClicked = false;
+
+        unhideAllColumns();
       }      
       virtual ~UIList() {_FreeUIList();}
 
@@ -588,12 +625,20 @@ namespace vapp {
       void addColumn(std::string Title,int nWidth) {m_Columns.push_back(Title); m_ColumnWidths.push_back(nWidth);}
       void setEnterButton(UIButton *pButton) {m_pEnterButton = pButton;}
       bool isItemActivated(void) {return m_bItemActivated;}
+      void setHideColumn(int n) {m_nColumnHideFlags |= (1<<n);}
+      void unhideAllColumns(void) {m_nColumnHideFlags=0;}
+      void setSort(bool bSort) {m_bSort=bSort;}
       
+      bool isClicked(void) {return m_bClicked;}
+      void setClicked(bool b) {m_bClicked=b;}
+
     private:
       /* Data */
+      bool m_bSort;
       std::vector<UIListEntry *> m_Entries;
       std::vector<std::string> m_Columns;
       std::vector<int> m_ColumnWidths;
+      unsigned int m_nColumnHideFlags;
       int m_nSelected;
       bool m_bItemActivated;
       UIButton *m_pEnterButton; /* if not null this is the "default" button of the list, i.e. the one 
@@ -603,6 +648,8 @@ namespace vapp {
       
       bool m_bScrollUpHover,m_bScrollDownHover;
       
+      bool m_bClicked;
+
       /* Helpers */
       void _FreeUIList(void);
       void _NewlySelectedItem(void);
@@ -619,6 +666,7 @@ namespace vapp {
         initW(pParent,x,y,Caption,nWidth,nHeight);        
         
         m_nSelected = 0;
+	m_bChanged = 0;
       }      
 
       /* Methods */
@@ -628,6 +676,9 @@ namespace vapp {
       virtual void mouseRDown(int x,int y);
       virtual void mouseRUp(int x,int y);
       virtual void mouseHover(int x,int y);      
+      virtual std::string subContextHelp(int x,int y);
+      
+      void setTabContextHelp(int nTab,const std::string &s);
       
       /* Data interface */      
       UIWindow *getSelected(void) {
@@ -635,10 +686,18 @@ namespace vapp {
           return getChildren()[m_nSelected];
         return NULL;
       }
+      void setSelected(int n) {
+        m_nSelected = n;
+      }
       
+      bool isChanged(void) {return m_bChanged;}
+      void setChanged(bool b) {m_bChanged=b;}
+
     private:
       /* Data */
       int m_nSelected;
+      bool m_bChanged;
+      std::vector<std::string> m_TabContextHelp;
   };
 
 	/*===========================================================================
@@ -668,7 +727,7 @@ namespace vapp {
   class UIRoot : public UIWindow {
     public:
       UIRoot() {
-        _InitWindow();
+        _InitWindow();        
       }
     
       /* Methods */
@@ -682,17 +741,24 @@ namespace vapp {
       virtual void mouseWheelUp(int x,int y);
       virtual void mouseWheelDown(int x,int y);
       virtual bool keyDown(int nKey,int nChar);      
-      virtual bool keyUp(int nKey);      
+      virtual bool keyUp(int nKey);  
       
       void deactivate(UIWindow *pWindow);
       
+      void enableContextMenuDrawing(bool b) {m_bShowContextMenu=b;}
+      void clearContext(void) {m_CurrentContextHelp = "";}
+      
       void activateUp(void);
       void activateDown(void);
       void activateLeft(void);
       void activateRight(void);
-      
+            
     private:
-      /* Helpers */
+      /* Data */
+      bool m_bShowContextMenu;
+      std::string m_CurrentContextHelp;
+            
+      /* Helpers */      
       bool _RootKeyEvent(UIWindow *pWindow,UIRootKeyEvent Event,int nKey,int nChar);
       bool _RootMouseEvent(UIWindow *pWindow,UIRootMouseEvent Event,int x,int y);
       void _RootPaint(int x,int y,UIWindow *pWindow,UIRect *pRect);
