@@ -1,23 +1,23 @@
 /*=============================================================================
-XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+  XMOTO
+  Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
-This file is part of XMOTO.
+  This file is part of XMOTO.
 
-XMOTO is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+  XMOTO is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
 
-XMOTO is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  XMOTO is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with XMOTO; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-=============================================================================*/
+  You should have received a copy of the GNU General Public License
+  along with XMOTO; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  =============================================================================*/
 
 /* 
  *  Game state serialization and magic
@@ -28,8 +28,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 namespace vapp {
 
   /*===========================================================================
-  Game state interpolation for smoother replays 
-  ===========================================================================*/
+    Game state interpolation for smoother replays 
+    ===========================================================================*/
   void MotoGame::interpolateGameState(SerializedBikeState *pA,SerializedBikeState *pB,SerializedBikeState *p,float t) {
     /* First of all inherit everything from A */
     memcpy(p,pA,sizeof(SerializedBikeState));
@@ -42,89 +42,320 @@ namespace vapp {
   }
   
   /*===========================================================================
-  Decoding of event stream
-  ===========================================================================*/
-  void MotoGame::unserializeGameEvents(DBuffer &Buffer) {
+    Decoding of event stream
+    ===========================================================================*/
+  void MotoGame::unserializeGameEvents(DBuffer *Buffer, std::vector<RecordedGameEvent *> *v_ReplayEvents) {
     /* Continue until buffer is empty */
     bool bError = false;
-    while(Buffer.numRemainingBytes() > sizeof(float) && !bError) {
+    while((*Buffer).numRemainingBytes() > sizeof(float) && !bError) {
       /* Get event time */
       float fEventTime;
-      Buffer >> fEventTime;
+      GameEvent Event;
+
+      (*Buffer) >> fEventTime;
       
       /* Get event type */
       GameEventType EventType;
-      Buffer >> EventType;
+      (*Buffer) >> EventType;
       
+      bool bIsOk = false;
+
       /* What now depends on event type */
       switch(EventType) {
-        case GAME_EVENT_ENTITY_DESTROYED:
-          GameEvent Event;
-          
-          /* Read entity name */
-          int n;
-          Buffer >> n;
-          if(n >= sizeof(Event.u.EntityDestroyed.cEntityID)) {
-            Log("** Warning ** : Entity name in replay too long, ignoring all events!");
-            bError = true;
-          }
-          else {
-            Buffer.readBuf(Event.u.EntityDestroyed.cEntityID,n);
-            Event.u.EntityDestroyed.cEntityID[n] = '\0';
+      case GAME_EVENT_ENTITY_DESTROYED:       
+	/* Read entity name */
+	int n;
+	(*Buffer) >> n;
+	if(n >= sizeof(Event.u.EntityDestroyed.cEntityID)) {
+	  Log("** Warning ** : Entity name in replay too long, ignoring all events!");
+	  bError = true;
+	}
+	else {
+	  (*Buffer).readBuf(Event.u.EntityDestroyed.cEntityID,n);
+	  Event.u.EntityDestroyed.cEntityID[n] = '\0';
             
-            /* Read entity type */
-            Buffer >> Event.u.EntityDestroyed.Type;
+	  /* Read entity type */
+	  (*Buffer) >> Event.u.EntityDestroyed.Type;
             
-            /* Read size and pos */
-            Buffer >> Event.u.EntityDestroyed.fSize;
-            Buffer >> Event.u.EntityDestroyed.fPosX;
-            Buffer >> Event.u.EntityDestroyed.fPosY;            
+	  /* Read size and pos */
+	  (*Buffer) >> Event.u.EntityDestroyed.fSize;
+	  (*Buffer) >> Event.u.EntityDestroyed.fPosX;
+	  (*Buffer) >> Event.u.EntityDestroyed.fPosY;            
             
-            /* Seems ok, add it */
-            RecordedGameEvent *p = new RecordedGameEvent;
-            p->fTime = fEventTime;
-            p->bPassed = false;
-            p->Event.Type = EventType;
-            p->Event.nSeq = 0;
-            memcpy(&p->Event.u,&Event.u,sizeof(Event.u));
-            m_ReplayEvents.push_back(p);
-          }
-          break;
-        default:
-          Log("** Warning ** : Failed to parse game events in replay, it will probably not play right!");
-          bError = true;
-          break;
+	  bIsOk = true;
+	}
+	break;
+
+      case GAME_EVENT_LUA_CALL_SETENTITYPOS:
+	{
+	  int n;
+	  (*Buffer) >> n;
+	  if(n >= sizeof(Event.u.LuaCallSetentitypos.cEntityID)) {
+	    Log("** Warning ** : Entity name in replay too long, ignoring all events!");
+	    bError = true;
+	  }
+	  else {
+	    (*Buffer).readBuf(Event.u.LuaCallSetentitypos.cEntityID,n);
+	    Event.u.LuaCallSetentitypos.cEntityID[n] = '\0';
+
+	    (*Buffer) >> Event.u.LuaCallSetentitypos.x;
+	    (*Buffer) >> Event.u.LuaCallSetentitypos.y;
+
+	    bIsOk = true;
+	  }
+	}
+	break;
+
+      case GAME_EVENT_LUA_CALL_CLEARMESSAGES:
+	{
+	  bIsOk = true;
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_PLACEINGAMEARROW:
+	{
+	  (*Buffer) >> Event.u.LuaCallPlaceingamearrow.x;
+	  (*Buffer) >> Event.u.LuaCallPlaceingamearrow.y;
+	  (*Buffer) >> Event.u.LuaCallPlaceingamearrow.angle;
+	  bIsOk = true;
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_PLACESCREENARROW:
+	{
+	  (*Buffer) >> Event.u.LuaCallPlacescreenarrow.x;
+	  (*Buffer) >> Event.u.LuaCallPlacescreenarrow.y;
+	  (*Buffer) >> Event.u.LuaCallPlacescreenarrow.angle;
+	  bIsOk = true;
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_HIDEARROW:
+	{
+	  bIsOk = true;
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_MESSAGE:
+	{
+	  int n;
+	  (*Buffer) >> n;
+	  if(n >= sizeof(Event.u.LuaCallMessage.cMessage)) {
+	    Log("** Warning ** : Message in replay too long, ignoring all events!");
+	    bError = true;
+	  }
+	  else {
+	    (*Buffer).readBuf(Event.u.LuaCallMessage.cMessage, n);
+	    Event.u.LuaCallMessage.cMessage[n] = '\0';
+	    bIsOk = true;
+	  }
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_MOVEBLOCK:
+	{
+	  int n;
+	  (*Buffer) >> n;
+	  if(n >= sizeof(Event.u.LuaCallMoveblock.cBlockID)) {
+	    Log("** Warning ** : Block name in replay too long, ignoring all events!");
+	    bError = true;
+	  }
+	  else {
+	    (*Buffer).readBuf(Event.u.LuaCallMoveblock.cBlockID,n);
+	    Event.u.LuaCallMoveblock.cBlockID[n] = '\0';
+
+	    (*Buffer) >> Event.u.LuaCallMoveblock.x;
+	    (*Buffer) >> Event.u.LuaCallMoveblock.y;
+
+	    bIsOk = true;
+	  }
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_SETBLOCKPOS:
+	{
+	  int n;
+	  (*Buffer) >> n;
+	  if(n >= sizeof(Event.u.LuaCallSetblockpos.cBlockID)) {
+	    Log("** Warning ** : Block name in replay too long, ignoring all events!");
+	    bError = true;
+	  }
+	  else {
+	    (*Buffer).readBuf(Event.u.LuaCallSetblockpos.cBlockID,n);
+	    Event.u.LuaCallSetblockpos.cBlockID[n] = '\0';
+
+	    (*Buffer) >> Event.u.LuaCallSetblockpos.x;
+	    (*Buffer) >> Event.u.LuaCallSetblockpos.y;
+
+	    bIsOk = true;
+	  }
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_SETGRAVITY:
+	{
+	  (*Buffer) >> Event.u.LuaCallSetgravity.x;
+	  (*Buffer) >> Event.u.LuaCallSetgravity.y;
+	  bIsOk = true;
+	}
+	break;
+	
+      case GAME_EVENT_LUA_CALL_SETPLAYERPOSITION:
+	{
+	  (*Buffer) >> Event.u.LuaCallSetplayerposition.x;
+	  (*Buffer) >> Event.u.LuaCallSetplayerposition.y;
+	  (*Buffer) >> Event.u.LuaCallSetplayerposition.bRight;
+	  bIsOk = true;
+	}
+	break;
+	
+      default:
+	Log("** Warning ** : Failed to parse game events in replay, it will probably not play right!");
+	bError = true;
+	break;
       }
+
+      if(bIsOk) {
+	/* Seems ok, add it */
+	RecordedGameEvent *p = new RecordedGameEvent;
+	p->fTime = fEventTime;
+	p->bPassed = false;
+	p->Event.Type = EventType;
+	p->Event.nSeq = 0;
+	memcpy(&p->Event.u,&Event.u,sizeof(Event.u));
+	v_ReplayEvents->push_back(p);
+      }
+
     }
   }
 
   /*===========================================================================
-  Encoding of event buffer 
-  ===========================================================================*/
+    Encoding of event buffer 
+    ===========================================================================*/
   void MotoGame::_SerializeGameEventQueue(DBuffer &Buffer,GameEvent *pEvent) {
     /* Note how we couldn't care less about most of the game events */    
     switch(pEvent->Type) {
-      case GAME_EVENT_ENTITY_DESTROYED:
-        {          
-          int i;
-          Buffer << getTime();
-          Buffer << pEvent->Type;
-          Buffer << (i=strlen(pEvent->u.EntityDestroyed.cEntityID));
-          Buffer.writeBuf(pEvent->u.EntityDestroyed.cEntityID,i);
-          Buffer << pEvent->u.EntityDestroyed.Type;
-          Buffer << pEvent->u.EntityDestroyed.fSize;
-          Buffer << pEvent->u.EntityDestroyed.fPosX;
-          Buffer << pEvent->u.EntityDestroyed.fPosY;
-        }
-        break;
+    case GAME_EVENT_ENTITY_DESTROYED:
+      {          
+	int i;
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << (i=strlen(pEvent->u.EntityDestroyed.cEntityID));
+	Buffer.writeBuf(pEvent->u.EntityDestroyed.cEntityID,i);
+	Buffer << pEvent->u.EntityDestroyed.Type;
+	Buffer << pEvent->u.EntityDestroyed.fSize;
+	Buffer << pEvent->u.EntityDestroyed.fPosX;
+	Buffer << pEvent->u.EntityDestroyed.fPosY;
+      }
+      break;
+
+    case GAME_EVENT_LUA_CALL_SETENTITYPOS:
+      {
+	int i;
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << (i=strlen(pEvent->u.LuaCallSetentitypos.cEntityID));
+	Buffer.writeBuf(pEvent->u.LuaCallSetentitypos.cEntityID,i);
+	Buffer << pEvent->u.LuaCallSetentitypos.x;
+	Buffer << pEvent->u.LuaCallSetentitypos.y;
+      }
+      break;
+
+    case GAME_EVENT_LUA_CALL_CLEARMESSAGES:
+      {
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_PLACEINGAMEARROW:
+      {
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << pEvent->u.LuaCallPlaceingamearrow.x;
+	Buffer << pEvent->u.LuaCallPlaceingamearrow.y;
+	Buffer << pEvent->u.LuaCallPlaceingamearrow.angle;
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_PLACESCREENARROW:
+      {
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << pEvent->u.LuaCallPlacescreenarrow.x;
+	Buffer << pEvent->u.LuaCallPlacescreenarrow.y;
+	Buffer << pEvent->u.LuaCallPlacescreenarrow.angle;
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_HIDEARROW:
+      {
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_MESSAGE:
+      {
+	int i;
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << (i=strlen(pEvent->u.LuaCallMessage.cMessage));
+	Buffer.writeBuf(pEvent->u.LuaCallMessage.cMessage,i);
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_MOVEBLOCK:
+      {
+	int i;
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << (i=strlen(pEvent->u.LuaCallMoveblock.cBlockID));
+	Buffer.writeBuf(pEvent->u.LuaCallMoveblock.cBlockID,i);
+	Buffer << pEvent->u.LuaCallMoveblock.x;
+	Buffer << pEvent->u.LuaCallMoveblock.y;
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_SETBLOCKPOS:
+      {
+	int i;
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << (i=strlen(pEvent->u.LuaCallSetblockpos.cBlockID));
+	Buffer.writeBuf(pEvent->u.LuaCallSetblockpos.cBlockID,i);
+	Buffer << pEvent->u.LuaCallSetblockpos.x;
+	Buffer << pEvent->u.LuaCallSetblockpos.y;
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_SETGRAVITY:
+      {
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << pEvent->u.LuaCallSetgravity.x;
+	Buffer << pEvent->u.LuaCallSetgravity.y;
+      }
+      break;
+      
+    case GAME_EVENT_LUA_CALL_SETPLAYERPOSITION:
+      {
+	Buffer << getTime();
+	Buffer << pEvent->Type;
+	Buffer << pEvent->u.LuaCallSetplayerposition.x;
+	Buffer << pEvent->u.LuaCallSetplayerposition.y;
+	Buffer << pEvent->u.LuaCallSetplayerposition.bRight;
+      }
+      break;
+      
     }            
   }
 
   /*===========================================================================
-  Matrix encodings
-  ===========================================================================*/
+    Matrix encodings
+    ===========================================================================*/
   unsigned short MotoGame::_MatrixTo16Bits(const float *pfMatrix) {
-    /* We idea is that we only need to store the first column of the matrix,
+    /* The idea is that we only need to store the first column of the matrix,
        as the second on is simply the first one transposed... Each of the
        two components of the first column is given 8 bits of precision */
     int n1 = (int)(pfMatrix[0] * 127.0f + 127.0f);   
@@ -163,8 +394,8 @@ namespace vapp {
   }
 
   /*===========================================================================
-  Neat trick for converting floating-point numbers to 8 bits
-  ===========================================================================*/
+    Neat trick for converting floating-point numbers to 8 bits
+    ===========================================================================*/
   char MotoGame::_MapCoordTo8Bits(float fRef,float fMaxDiff,float fCoord) {
     int n = (int)((127.0f * (fCoord-fRef))/fMaxDiff);
     if(n<-127) n=-127;
@@ -177,8 +408,8 @@ namespace vapp {
   }
 
   /*===========================================================================
-  Serializer
-  ===========================================================================*/
+    Serializer
+    ===========================================================================*/
   void MotoGame::getSerializedBikeState(SerializedBikeState *pState) {
     /* Get. */
     pState->fGameTime = m_fTime;
