@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -35,7 +35,12 @@ namespace vapp {
     "Brake",           ACTION_BRAKE,             
     "PullBack",        ACTION_PULLBACK,          
     "PushForward",     ACTION_PUSHFORWARD,       
-    "ChangeDirection", ACTION_CHANGEDIR,         
+    "ChangeDirection", ACTION_CHANGEDIR,     
+    #if defined(ENABLE_ZOOMING)    
+      "ZoomIn",        ACTION_ZOOMIN,     
+      "ZoomOut",       ACTION_ZOOMOUT,
+      "ZoomInit",      ACTION_ZOOMINIT, 
+    #endif
     NULL
   };
 
@@ -82,6 +87,16 @@ namespace vapp {
     "8",               SDLK_8,
     "9",               SDLK_9,
     "0",               SDLK_0,
+    "PageUp",          SDLK_PAGEUP,
+    "PageDown",        SDLK_PAGEDOWN,
+    "Home",            SDLK_HOME,
+    "End",             SDLK_END,
+    "Return",          SDLK_RETURN,
+    "Left Click",      SDL_BUTTON_LEFT,
+    "Right Click",     SDL_BUTTON_RIGHT,
+    "Middle Click",    SDL_BUTTON_MIDDLE,
+    "Wheel down",      SDL_BUTTON_WHEELDOWN,
+    "Wheel up",        SDL_BUTTON_WHEELUP,
     /* TODO: add more */
     NULL    
   };
@@ -265,7 +280,11 @@ namespace vapp {
         switch(Event.type) {
           case SDL_QUIT:
             return "<<QUIT>>";
-          case SDL_KEYDOWN:
+	case SDL_MOUSEBUTTONDOWN:
+	  Ret = _KeyToString(Event.button.button);
+	  if(Ret != "") bWait = false;
+	  break;
+	case SDL_KEYDOWN:
             if(Event.key.keysym.sym == SDLK_ESCAPE) return "<<CANCEL>>";
           
             Ret = _KeyToString(Event.key.keysym.sym);
@@ -303,12 +322,22 @@ namespace vapp {
         m_nPullBackKey1 = _StringToKey(pConfig->getString("KeyFlipLeft1"));
         m_nPushForwardKey1 = _StringToKey(pConfig->getString("KeyFlipRight1"));
         m_nChangeDirKey1 = _StringToKey(pConfig->getString("KeyChangeDir1"));
+
+        #if defined(ENABLE_ZOOMING)          
+	m_nZoomIn   = _StringToKey(pConfig->getString("KeyZoomIn"));
+	m_nZoomOut  = _StringToKey(pConfig->getString("KeyZoomOut"));
+	m_nZoomInit = _StringToKey(pConfig->getString("KeyZoomInit"));
+	#endif
         
         /* All good? */
         if(m_nDriveKey1<0 || m_nBrakeKey1<0 || m_nPullBackKey1<0 ||
-          m_nPushForwardKey1<0 || m_nChangeDirKey1<0) {
+          m_nPushForwardKey1<0 || m_nChangeDirKey1<0 
+          #if defined(ENABLE_ZOOMING)
+            ||m_nZoomIn<0 || m_nZoomOut <0 || m_nZoomInit <0
+          #endif
+	        ) {
           Log("** Warning ** : Invalid keyboard configuration!");
-          _SetDefaultConfig();
+	  _SetDefaultConfigToUnsetKeys();
         }
         break;
       }
@@ -345,11 +374,26 @@ namespace vapp {
       }
     }      
   }
+  
+  /*===========================================================================
+  Add script key hook
+  ===========================================================================*/  
+  void InputHandler::addScriptKeyHook(MotoGame *pGame,const std::string &KeyName,const std::string &FuncName) {
+    if(m_nNumScriptKeyHooks < MAX_SCRIPT_KEY_HOOKS) {
+      m_ScriptKeyHooks[m_nNumScriptKeyHooks].FuncName = FuncName;
+      m_ScriptKeyHooks[m_nNumScriptKeyHooks].nKey = _StringToKey(KeyName);
+      m_ScriptKeyHooks[m_nNumScriptKeyHooks].pGame = pGame;
+      m_nNumScriptKeyHooks++;
+    }
+  }  
 
   /*===========================================================================
   Handle an input event
   ===========================================================================*/  
-  void InputHandler::handleInput(InputEventType Type,int nKey,BikeController *pController) {
+  void InputHandler::handleInput(InputEventType Type,
+				 int nKey,
+				 BikeController *pController,
+				 GameRenderer *pGameRender) {
     /* Update controller 1 */
     if(m_ControllerModeID1 == CONTROLLER_MODE_KEYBOARD) {
       /* Keyboard controlled */
@@ -370,7 +414,22 @@ namespace vapp {
           else if(m_nPushForwardKey1 == nKey) {
             /* Push forward */
             pController->bPushForward = true;            
-          }
+          } 
+#if defined(ENABLE_ZOOMING)          
+	else if(m_nZoomIn == nKey) {
+	  /* Zoom in */
+	  pGameRender->zoom(0.002);
+	} 
+	else if(m_nZoomOut == nKey) {
+	  /* Zoom out */
+	  pGameRender->zoom(-0.002);
+	}
+	else if(m_nZoomInit == nKey) {
+	  /* Zoom out */
+	  pGameRender->initZoom();
+	}
+#endif
+
           break;
         case INPUT_KEY_UP:
           if(m_nDriveKey1 == nKey) {
@@ -396,6 +455,16 @@ namespace vapp {
           break;
       }      
     }
+    
+    /* Have the script hooked this key? */
+    if(Type == INPUT_KEY_DOWN) {
+      for(int i=0;i<m_nNumScriptKeyHooks;i++) {
+        if(m_ScriptKeyHooks[i].nKey == nKey) {
+          /* Invoke script */
+          m_ScriptKeyHooks[i].pGame->scriptCallVoid(m_ScriptKeyHooks[i].FuncName);
+        }
+      }
+    }
   }
   
   /*===========================================================================
@@ -404,13 +473,55 @@ namespace vapp {
   void InputHandler::_SetDefaultConfig(void) {
     m_ControllerModeID1 = CONTROLLER_MODE_KEYBOARD;
     
-    m_nDriveKey1 = SDLK_UP;
-    m_nBrakeKey1 = SDLK_DOWN;
-    m_nPullBackKey1 = SDLK_LEFT;
+    m_nDriveKey1       = SDLK_UP;
+    m_nBrakeKey1       = SDLK_DOWN;
+    m_nPullBackKey1    = SDLK_LEFT;
     m_nPushForwardKey1 = SDLK_RIGHT;
-    m_nChangeDirKey1 = SDLK_SPACE;
+    m_nChangeDirKey1   = SDLK_SPACE;
+    
+    #if defined(ENABLE_ZOOMING)    
+      m_nZoomIn          = SDLK_PAGEUP;
+      m_nZoomOut         = SDLK_PAGEDOWN;
+      m_nZoomOut         = SDLK_HOME;
+    #endif
   }  
 
+  void InputHandler::_SetDefaultConfigToUnsetKeys() {
+    m_ControllerModeID1 = CONTROLLER_MODE_KEYBOARD;
+    
+    if(m_nDriveKey1       < 0) { m_nDriveKey1 	    = SDLK_UP;       }
+    if(m_nBrakeKey1       < 0) { m_nBrakeKey1 	    = SDLK_DOWN;     }
+    if(m_nPullBackKey1    < 0) { m_nPullBackKey1    = SDLK_LEFT;     }
+    if(m_nPushForwardKey1 < 0) { m_nPushForwardKey1 = SDLK_RIGHT;    }
+    if(m_nChangeDirKey1   < 0) { m_nChangeDirKey1   = SDLK_SPACE;    }
+    
+    #if defined(ENABLE_ZOOMING)
+    if(m_nZoomIn          < 0) { m_nZoomIn          = SDLK_PAGEUP;   }
+    if(m_nZoomOut         < 0) { m_nZoomOut         = SDLK_PAGEDOWN; }
+    if(m_nZoomInit        < 0) { m_nZoomInit        = SDLK_HOME; }
+    #endif
+  }
+
+  /*===========================================================================
+  Get key by action...
+  ===========================================================================*/  
+  std::string InputHandler::getKeyByAction(const std::string &Action) {
+    if(m_ControllerModeID1 != CONTROLLER_MODE_KEYBOARD) return "?";
+    
+    if(Action == "Drive") return _KeyToString(m_nDriveKey1);
+    if(Action == "Brake") return _KeyToString(m_nBrakeKey1);
+    if(Action == "PullBack") return _KeyToString(m_nPullBackKey1);
+    if(Action == "PushForward") return _KeyToString(m_nPushForwardKey1);
+    if(Action == "ChangeDir") return _KeyToString(m_nChangeDirKey1);
+    
+    #if defined(ENABLE_ZOOMING)    
+    if(Action == "ZoomIn")   return _KeyToString(m_nZoomIn);
+    if(Action == "ZoomOut")  return _KeyToString(m_nZoomOut);
+    if(Action == "ZoomInit") return _KeyToString(m_nZoomInit);
+    #endif
+
+    return "?";
+  }
 
 };
 
