@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -77,7 +77,11 @@ namespace vapp {
     }
     
     FS::writeLineF(pfh,"<?xml version=\"1.0\" encoding=\"utf-8\"?>");
-    FS::writeLineF(pfh,"<level id=\"%s\">",m_ID.c_str());
+    
+    if(m_LevelPack != "")
+      FS::writeLineF(pfh,"<level id=\"%s\" levelpack=\"%s\">",m_ID.c_str(),m_LevelPack.c_str());
+    else
+      FS::writeLineF(pfh,"<level id=\"%s\">",m_ID.c_str());
     
     /* INFO */
     FS::writeLineF(pfh,"\t<info>");
@@ -89,8 +93,35 @@ namespace vapp {
     FS::writeLineF(pfh,"\t</info>");
     
     /* MISC */
-    if(m_ScriptFile != "")
+    if(m_ScriptFile != "" && m_ScriptSource != "") {
+      FS::writeLineF(pfh,"\t<script source=\"%s\">",m_ScriptFile.c_str());
+      FS::writeByte(pfh,'\t'); FS::writeByte(pfh,'\t');
+      
+      /* Write script and transform dangerous characters */
+      for(int i=0;i<m_ScriptSource.length();i++) {
+        int c = m_ScriptSource[i];
+        if(c == '<') FS::writeBuf(pfh,"&lt;",4);
+        else if(c == '>') FS::writeBuf(pfh,"&gt;",4);
+        else FS::writeByte(pfh,c);
+      }
+      //FS::writeBuf(pfh,(char *)m_ScriptSource.c_str(),m_ScriptSource.length());
+      FS::writeLineF(pfh,"</script>");
+    }
+    else if(m_ScriptFile != "")
       FS::writeLineF(pfh,"\t<script source=\"%s\"/>",m_ScriptFile.c_str());
+    else if(m_ScriptSource != "") {
+      FS::writeLineF(pfh,"\t<script>");
+      FS::writeByte(pfh,'\t'); FS::writeByte(pfh,'\t');
+      /* Write script and transform dangerous characters */
+      for(int i=0;i<m_ScriptSource.length();i++) {
+        int c = m_ScriptSource[i];
+        if(c == '<') FS::writeBuf(pfh,"&lt;",4);
+        else if(c == '>') FS::writeBuf(pfh,"&gt;",4);
+        else FS::writeByte(pfh,c);
+      }
+//      FS::writeBuf(pfh,(char *)m_ScriptSource.c_str(),m_ScriptSource.length());
+      FS::writeLineF(pfh,"</script>");
+    }
       
     FS::writeLineF(pfh,"\t<limits left=\"%f\" right=\"%f\" top=\"%f\" bottom=\"%f\"/>",m_fLeftLimit,m_fRightLimit,m_fTopLimit,m_fBottomLimit);
     
@@ -150,7 +181,8 @@ namespace vapp {
   void LevelSrc::loadXML(void) {
     /* Load XML document and fetch tinyxml handle */
     _UnloadLevelData();
-    m_XML.readFromFile( m_FileName );
+    m_LevelCheckSum.nCRC32 = 0;
+    m_XML.readFromFile( m_FileName, /*&m_LevelCheckSum.nCRC32*/ NULL );
     
     TiXmlDocument *pDoc = m_XML.getLowLevelAccess();
     
@@ -162,6 +194,9 @@ namespace vapp {
     m_ID = _GetOption(pLevelElem,"id");
     if(m_ID == "") return; /* TODO: error */    
     
+    /* Get level pack */
+    m_LevelPack = _GetOption(pLevelElem,"levelpack");
+    
     /* Set default info */
     m_Info.Name = m_FileName;
     m_Info.Date = "";
@@ -193,11 +228,22 @@ namespace vapp {
       if(Tmp != "") m_Info.Sky = Tmp;
     }
     
-    /* Get script file */
+    /* Get script */
     m_ScriptFile = "";
+    m_ScriptSource = "";
+    
     TiXmlElement *pScriptElem = _FindElement(pLevelElem,std::string("script"));
     if(pScriptElem != NULL) {
+      /* External script file specified? */
       m_ScriptFile = _GetOption(pScriptElem,"source");      
+      
+      /* Encapsulated script? */
+      for(TiXmlNode *pScript=pScriptElem->FirstChild();pScript!=NULL;
+          pScript=pScript->NextSibling()) {
+        if(pScript->Type() == TiXmlNode::TEXT) {
+          m_ScriptSource.append(pScript->Value());
+        }
+      }
     }    
     
     /* Get level limits */
@@ -514,6 +560,293 @@ namespace vapp {
     if(pc == NULL) return Default;
     return std::string(pc);
   }
+  
+  /*===========================================================================
+  Calculate checksum of level XML before loading it
+  ===========================================================================*/
+  bool LevelSrc::probeCheckSum(LevelCheckSum *pSum) {
+    m_XML.readFromFile( m_FileName,&pSum->nCRC32 );      
+    return true;
+  }
+      
+  /*===========================================================================
+  Export binary level file
+  ===========================================================================*/
+  void LevelSrc::exportBinary(const std::string &FileName,LevelCheckSum *pSum) {
+    /* Export binary... */
+    FileHandle *pfh = FS::openOFile(FileName);
+    if(pfh == NULL) {
+      Log("** Warning ** : Failed to export binary: %s",FileName.c_str());
+    }
+    else {
+      /* Write tag */
+      FS::writeBuf(pfh,"XBL1",4);
+      
+      /* Write CRC32 of XML */
+      FS::writeInt_LE(pfh,pSum->nCRC32);
+         
+      //if(m_ID == "") {
+      //  __asm {int 3};
+      //}
+         
+      /* Write header */
+      FS::writeString(pfh,m_ID);
+      FS::writeString(pfh,m_LevelPack);
+      FS::writeString(pfh,m_Info.Name);
+      FS::writeString(pfh,m_Info.Description);
+      FS::writeString(pfh,m_Info.Author);
+      FS::writeString(pfh,m_Info.Date);
+      FS::writeString(pfh,m_Info.Sky);
+      FS::writeString(pfh,m_ScriptFile);
+      
+      FS::writeFloat(pfh,m_fLeftLimit);
+      FS::writeFloat(pfh,m_fRightLimit);
+      FS::writeFloat(pfh,m_fTopLimit);
+      FS::writeFloat(pfh,m_fBottomLimit);
+      
+      /* Write script (if any) */
+      FS::writeInt_LE(pfh,m_ScriptSource.length());
+      FS::writeBuf(pfh,(char *)m_ScriptSource.c_str(),m_ScriptSource.length());
+      
+      /* Write blocks */
+      FS::writeInt_LE(pfh,m_Blocks.size());
+      for(int i=0;i<m_Blocks.size();i++) {
+        FS::writeString(pfh,m_Blocks[i]->ID);
+        FS::writeBool(pfh,m_Blocks[i]->bBackground);
+        FS::writeBool(pfh,m_Blocks[i]->bWater);
+        FS::writeString(pfh,m_Blocks[i]->Texture);
+        FS::writeFloat(pfh,m_Blocks[i]->fPosX);
+        FS::writeFloat(pfh,m_Blocks[i]->fPosY);
+        
+        FS::writeShort_LE(pfh,m_Blocks[i]->Vertices.size());
+        
+        for(int j=0;j<m_Blocks[i]->Vertices.size();j++) {
+          FS::writeFloat(pfh,m_Blocks[i]->Vertices[j]->fX);
+          FS::writeFloat(pfh,m_Blocks[i]->Vertices[j]->fY);
+          FS::writeString(pfh,m_Blocks[i]->Vertices[j]->EdgeEffect);
+        }        
+      }
+      
+      /* Write entities */
+      FS::writeInt_LE(pfh,m_Entities.size());
+      for(int i=0;i<m_Entities.size();i++) {
+			  FS::writeString(pfh,m_Entities[i]->ID);
+			  FS::writeString(pfh,m_Entities[i]->TypeID);
+			  FS::writeFloat(pfh,m_Entities[i]->fSize);
+        FS::writeFloat(pfh,m_Entities[i]->fPosX);
+        FS::writeFloat(pfh,m_Entities[i]->fPosY);
+        FS::writeByte(pfh,m_Entities[i]->Params.size());       
+        for(int j=0;j<m_Entities[i]->Params.size();j++) {
+          FS::writeString(pfh,m_Entities[i]->Params[j]->Name);
+          FS::writeString(pfh,m_Entities[i]->Params[j]->Value);        
+        }
+      }  
+      
+      /* Write zones */
+      FS::writeInt_LE(pfh,m_Zones.size());
+      for(int i=0;i<m_Zones.size();i++) {
+        FS::writeString(pfh,m_Zones[i]->ID);
+        FS::writeByte(pfh,m_Zones[i]->Prims.size());
+        
+        for(int j=0;j<m_Zones[i]->Prims.size();j++) {
+          FS::writeInt_LE(pfh,(int)m_Zones[i]->Prims[j]->Type);
+        
+          if(m_Zones[i]->Prims[j]->Type == LZPT_BOX) {
+            FS::writeFloat(pfh,m_Zones[i]->Prims[j]->fLeft);
+            FS::writeFloat(pfh,m_Zones[i]->Prims[j]->fRight);
+            FS::writeFloat(pfh,m_Zones[i]->Prims[j]->fTop);
+            FS::writeFloat(pfh,m_Zones[i]->Prims[j]->fBottom);
+          }
+        }
+      }
+                
+      /* clean up */
+      FS::closeFile(pfh);
+    }
+  }
+  
+  /*===========================================================================
+  Import binary level file
+  ===========================================================================*/
+  bool LevelSrc::importBinary(const std::string &FileName,LevelCheckSum *pSum) {
+    _UnloadLevelData();
+    bool bRet = true;
 
+    m_fPlayerStartX = m_fPlayerStartY = 0.0f;
+
+    /* Import binary */
+    FileHandle *pfh = FS::openIFile(FileName);
+    if(pfh == NULL) {
+      return false;
+    }
+    else {
+      /* Read tag - it tells something about the format */
+      char cTag[5];
+      FS::readBuf(pfh,(char *)cTag,4);
+      cTag[4] = '\0';
+      int nFormat = 0;
+      if(!strcmp(cTag,"XBL1"))
+        nFormat = 1;
+        
+      if(nFormat == 1) {
+        /* Read "format 1" binary level */
+        m_LevelCheckSum.nCRC32 = pSum->nCRC32;
+        
+        /* Right CRC? */
+        if(FS::readInt_LE(pfh) != pSum->nCRC32) {
+          Log("** Warning ** : CRC check failed, can't import: %s",FileName.c_str());
+          bRet = false;
+        }
+        else {
+          /* Read header */
+          m_ID = FS::readString(pfh);
+          m_LevelPack = FS::readString(pfh);
+          m_Info.Name = FS::readString(pfh);
+          m_Info.Description = FS::readString(pfh);
+          m_Info.Author = FS::readString(pfh);
+          m_Info.Date = FS::readString(pfh);
+          m_Info.Sky = FS::readString(pfh);
+          m_ScriptFile = FS::readString(pfh);
+
+          //m_fLeftLimit = FS::readFloat(pfh);
+          //m_fRightLimit = FS::readFloat(pfh);
+          //m_fTopLimit = FS::readFloat(pfh);
+          //m_fBottomLimit = FS::readFloat(pfh);
+
+          m_fLeftLimit = FS::readFloat(pfh);
+          m_fRightLimit = FS::readFloat(pfh);
+          m_fTopLimit = FS::readFloat(pfh);
+          m_fBottomLimit = FS::readFloat(pfh);
+
+          /* Read embedded script */
+          int nScriptSourceLen = FS::readInt_LE(pfh);
+          if(nScriptSourceLen > 0) {
+            char *pcTemp = new char[nScriptSourceLen+1];
+            FS::readBuf(pfh,(char *)pcTemp,nScriptSourceLen);
+            pcTemp[nScriptSourceLen]='\0';
+            
+            m_ScriptSource = pcTemp;
+            
+            delete [] pcTemp;           
+          }
+          else
+            m_ScriptSource = "";
+
+          /* Read blocks */
+          int nNumBlocks = FS::readInt_LE(pfh);
+          m_Blocks.reserve(nNumBlocks);
+          for(int i=0;i<nNumBlocks;i++) {
+            LevelBlock *pBlock = new LevelBlock;
+            pBlock->ID = FS::readString(pfh);
+            pBlock->bBackground = FS::readBool(pfh);
+            pBlock->bWater = FS::readBool(pfh);
+            pBlock->Texture = FS::readString(pfh);
+            pBlock->fPosX = FS::readFloat(pfh);
+            pBlock->fPosY = FS::readFloat(pfh);
+            
+            int nNumVertices = FS::readShort_LE(pfh);
+            pBlock->Vertices.reserve(nNumVertices);
+            for(int j=0;j<nNumVertices;j++) {
+              LevelBlockVertex *pV = new LevelBlockVertex;
+              pV->fX = FS::readFloat(pfh);
+              pV->fY = FS::readFloat(pfh);
+              pV->EdgeEffect = FS::readString(pfh);
+                            
+              pBlock->Vertices.push_back(pV);
+            }
+            
+            m_Blocks.push_back(pBlock);
+          }
+
+          /* Read entities */
+          int nNumEntities = FS::readInt_LE(pfh);
+          m_Entities.reserve(nNumEntities);
+          for(int i=0;i<nNumEntities;i++) {
+            LevelEntity *pEnt = new LevelEntity;
+            pEnt->ID = FS::readString(pfh);
+            pEnt->TypeID = FS::readString(pfh);
+            pEnt->fSize = FS::readFloat(pfh);
+            pEnt->fPosX = FS::readFloat(pfh);
+            pEnt->fPosY = FS::readFloat(pfh);
+            
+            int nNumParams = FS::readByte(pfh);
+            pEnt->Params.reserve(nNumParams);
+            for(int j=0;j<nNumParams;j++) {
+              LevelEntityParam *pP = new LevelEntityParam;
+              pP->Name = FS::readString(pfh);
+              pP->Value = FS::readString(pfh);
+              
+              pEnt->Params.push_back(pP);
+            }
+            
+            m_Entities.push_back(pEnt);
+            
+            /* Player start? */
+            if(pEnt->TypeID == "PlayerStart") {
+              m_fPlayerStartX = pEnt->fPosX;
+              m_fPlayerStartY = pEnt->fPosY;
+            }
+          }
+          
+          /* Read zones */
+          int nNumZones = FS::readInt_LE(pfh);
+          m_Zones.reserve(nNumZones);
+          for(int i=0;i<nNumZones;i++) {
+            LevelZone *pZone = new LevelZone;
+            pZone->ID = FS::readString(pfh);
+            
+            int nNumPrims = FS::readByte(pfh);
+            pZone->Prims.reserve(nNumPrims);
+            for(int j=0;j<nNumPrims;j++) {
+              LevelZonePrim *pP = new LevelZonePrim;
+              pP->Type = (LevelZonePrimType)FS::readInt_LE(pfh);
+              
+              if(pP->Type == LZPT_BOX) {
+                pP->fLeft = FS::readFloat(pfh);
+                pP->fRight = FS::readFloat(pfh);
+                pP->fTop = FS::readFloat(pfh);
+                pP->fBottom = FS::readFloat(pfh);
+              }
+              else {
+                Log("** Warning ** : Invalid zone primitive encountered in: %s",FileName.c_str());
+                delete pP;
+                delete pZone;
+                bRet = false;
+                break;
+              }
+              
+              pZone->Prims.push_back(pP);
+            }
+            
+            if(!bRet) break;
+            
+            m_Zones.push_back(pZone);
+          }                                                                       
+        }
+      }
+      else {
+        Log("** Warning ** : Invalid binary format (%d), can't import: %s",nFormat,FileName.c_str());
+        bRet = false;
+      }
+             
+      /* clean up */
+      FS::closeFile(pfh);
+    }
+    
+    return bRet;
+  }
+
+  int LevelSrc::compareLevel(const LevelSrc *p_lvl1, const LevelSrc *p_lvl2) {
+    if(p_lvl1->m_Info.Name == p_lvl2->m_Info.Name) {
+      return 0;
+    }
+    
+    if(p_lvl1->m_Info.Name > p_lvl2->m_Info.Name) {
+      return 1;
+    } 
+
+    return -1;
+  }
+    
 };
 
