@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -31,11 +31,15 @@ namespace vapp {
   Init physics
   ===========================================================================*/
   void MotoGame::_InitPhysics(void) {
+    m_bFirstPhysicsUpdate = true;
+  
     /* Setup ODE */
     m_WorldID = dWorldCreate();
     dWorldSetERP(m_WorldID,PHYS_WORLD_ERP);
     dWorldSetCFM(m_WorldID,PHYS_WORLD_CFM);
-    dWorldSetGravity(m_WorldID,0,PHYS_WORLD_GRAV,0);    
+        
+    dWorldSetGravity(m_WorldID,m_PhysGravity.x,m_PhysGravity.y,0);    
+    
     m_ContactGroup = dJointGroupCreate(0);
     
     dWorldSetQuickStepNumIterations(m_WorldID,PHYS_QSTEP_ITERS);
@@ -84,6 +88,20 @@ namespace vapp {
     return m_BikeS.fBikeEngineRPM;
   }
 
+  float MotoGame::getBikeEngineSpeed() {
+    float fWheelAngVel;
+    float speed;
+
+    if(m_BikeS.Dir == DD_RIGHT) {
+      fWheelAngVel = dBodyGetAngularVel(m_RearWheelBodyID)[2];
+    } else {
+      fWheelAngVel = dBodyGetAngularVel(m_FrontWheelBodyID)[2];
+    }
+
+    speed = (fWheelAngVel * PHYS_WHEEL_RADIUS * 3.6);
+    return speed >= 0.0 ? speed : -speed;
+  }
+
   /*===========================================================================
   Update physics
   ===========================================================================*/
@@ -91,6 +109,9 @@ namespace vapp {
     /* No wheel spin per default */
     m_bWheelSpin = false;
 
+    /* Update gravity vector */
+    dWorldSetGravity(m_WorldID,m_PhysGravity.x,m_PhysGravity.y,0);    
+
     /* Should we disable stuff? Ok ODE has an autodisable feature, but i'd rather
       roll my own here. */
     const dReal *pfFront = dBodyGetLinearVel(m_FrontWheelBodyID);
@@ -107,7 +128,7 @@ namespace vapp {
     }
     bool bSleep = false;
     
-    //printf("[%d]",m_nStillFrames);        
+    //printf("]",m_nStillFrames);        
     if(m_nStillFrames > PHYS_SLEEP_FRAMES) {
       bSleep = true;
       dBodyDisable(m_FrontWheelBodyID);
@@ -119,6 +140,8 @@ namespace vapp {
       if(!dBodyIsEnabled(m_RearWheelBodyID)) dBodyEnable(m_RearWheelBodyID);
       if(!dBodyIsEnabled(m_FrameBodyID)) dBodyEnable(m_FrameBodyID);
     }
+    
+    //printf("%d",bSleep);
 
     if(!bSleep) {
       /* Update front suspension */
@@ -257,7 +280,7 @@ namespace vapp {
     static int nSlipFrameInterlace =0; /* Okay, lazy approach again. Front wheel can generate particles
                                           even frames, rear wheel odd frames */
     nSlipFrameInterlace++;
-    
+        
     if(dBodyIsEnabled(m_FrontWheelBodyID)) {
       nNumContacts = _IntersectWheelLevel( m_BikeS.FrontWheelP,m_BikeP.WR,Contacts );
       Vector2f WSP;
@@ -340,11 +363,11 @@ namespace vapp {
           m_WheelSpinDir.y = (((m_BikeS.RearWheelP.x - WSP.x))*1 + (m_BikeS.RearWheelP.y - WSP.y)) /2;
         }
       }
-    }
+    }        
     
     /* Player head */
     if(m_BikeS.Dir == DD_RIGHT) {
-      if(_IntersectHeadLevel(m_BikeS.HeadP,m_BikeP.fHeadSize)) {
+      if(_IntersectHeadLevel(m_BikeS.HeadP,m_BikeP.fHeadSize,m_PrevHeadP)) {
 				GameEvent *pEvent = createGameEvent(GAME_EVENT_PLAYER_DIES);
 				if(pEvent != NULL) {
 					pEvent->u.PlayerDies.bWrecker = false;
@@ -352,7 +375,7 @@ namespace vapp {
       }
     }
     else if(m_BikeS.Dir == DD_LEFT) {
-      if(_IntersectHeadLevel(m_BikeS.Head2P,m_BikeP.fHeadSize)) {
+      if(_IntersectHeadLevel(m_BikeS.Head2P,m_BikeP.fHeadSize,m_PrevHead2P)) {
 				GameEvent *pEvent = createGameEvent(GAME_EVENT_PLAYER_DIES);
 				if(pEvent != NULL) {
 					pEvent->u.PlayerDies.bWrecker = false;
@@ -360,6 +383,11 @@ namespace vapp {
       }
     }
     
+    //m_PrevFrontWheelP = m_BikeS.FrontWheelP;
+    //m_PrevRearWheelP = m_BikeS.RearWheelP;
+    m_PrevHeadP = m_BikeS.HeadP;
+    m_PrevHead2P = m_BikeS.Head2P;
+    
     /* Move rider along bike -- calculate handlebar and footpeg coords */
     Vector2f FootRP,HandRP;
     
@@ -407,11 +435,115 @@ namespace vapp {
     
     /* Empty contact joint group */
     dJointGroupEmpty(m_ContactGroup);
+    
+    m_bFirstPhysicsUpdate = false;
   }
 
+
+#if defined(ALLOW_GHOST)
   /*===========================================================================
-  Update game state
-  ===========================================================================*/
+    Update game state
+    ===========================================================================*/
+  void MotoGame::_UpdateStateFromReplay(SerializedBikeState *pReplayState,
+					BikeState *pBikeS) 
+  {
+    bool bUpdateRider=true,bUpdateAltRider=true;
+    
+    /* Replaying... fetch serialized state */
+    pBikeS->RearWheelP.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cRearWheelX); 
+    pBikeS->RearWheelP.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cRearWheelY);
+    pBikeS->FrontWheelP.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cFrontWheelX); 
+    pBikeS->FrontWheelP.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cFrontWheelY);
+    pBikeS->CenterP.x = pReplayState->fFrameX; 
+    pBikeS->CenterP.y = pReplayState->fFrameY;
+    
+    _16BitsToMatrix(pReplayState->nFrameRot,pBikeS->fFrameRot);
+    _16BitsToMatrix(pReplayState->nFrontWheelRot,pBikeS->fFrontWheelRot);
+    _16BitsToMatrix(pReplayState->nRearWheelRot,pBikeS->fRearWheelRot);
+    
+    /* Update engine stuff */
+    pBikeS->fBikeEngineRPM = ENGINE_MIN_RPM + (ENGINE_MAX_RPM - ENGINE_MIN_RPM) * ((float)pReplayState->cBikeEngineRPM) / 255.0f;
+    
+    pBikeS->SwingAnchorP.x = m_BikeA.AR.x*pBikeS->fFrameRot[0] + m_BikeA.AR.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->SwingAnchorP.y = m_BikeA.AR.x*pBikeS->fFrameRot[2] + m_BikeA.AR.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
+    pBikeS->FrontAnchorP.x = m_BikeA.AF.x*pBikeS->fFrameRot[0] + m_BikeA.AF.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->FrontAnchorP.y = m_BikeA.AF.x*pBikeS->fFrameRot[2] + m_BikeA.AF.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
+    
+    pBikeS->SwingAnchor2P.x = m_BikeA.AR2.x*pBikeS->fFrameRot[0] + m_BikeA.AR2.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->SwingAnchor2P.y = m_BikeA.AR2.x*pBikeS->fFrameRot[2] + m_BikeA.AR2.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
+    pBikeS->FrontAnchor2P.x = m_BikeA.AF2.x*pBikeS->fFrameRot[0] + m_BikeA.AF2.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->FrontAnchor2P.y = m_BikeA.AF2.x*pBikeS->fFrameRot[2] + m_BikeA.AF2.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
+    
+    /* Calculate desired hand/foot positions */
+    pBikeS->WantedFootP.x = m_BikeA.PFp.x*pBikeS->fFrameRot[0] + m_BikeA.PFp.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->WantedFootP.y = m_BikeA.PFp.x*pBikeS->fFrameRot[2] + m_BikeA.PFp.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
+    pBikeS->WantedHandP.x = m_BikeA.PHp.x*pBikeS->fFrameRot[0] + m_BikeA.PHp.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->WantedHandP.y = m_BikeA.PHp.x*pBikeS->fFrameRot[2] + m_BikeA.PHp.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;    
+    
+    pBikeS->WantedFoot2P.x = m_BikeA.PFp2.x*pBikeS->fFrameRot[0] + m_BikeA.PFp2.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->WantedFoot2P.y = m_BikeA.PFp2.x*pBikeS->fFrameRot[2] + m_BikeA.PFp2.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
+    pBikeS->WantedHand2P.x = m_BikeA.PHp2.x*pBikeS->fFrameRot[0] + m_BikeA.PHp2.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->WantedHand2P.y = m_BikeA.PHp2.x*pBikeS->fFrameRot[2] + m_BikeA.PHp2.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;    
+    
+    /* Get hinges from serialized state */
+    if(pReplayState->cFlags & SER_BIKE_STATE_DIR_RIGHT) {
+      pBikeS->HandP = pBikeS->WantedHandP;
+      pBikeS->ElbowP.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cElbowX);         
+      pBikeS->ElbowP.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cElbowY);
+      pBikeS->ShoulderP.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cShoulderX); 
+      pBikeS->ShoulderP.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cShoulderY);
+      pBikeS->LowerBodyP.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cLowerBodyX); 
+      pBikeS->LowerBodyP.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cLowerBodyY);
+      pBikeS->KneeP.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cKneeX); 
+      pBikeS->KneeP.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cKneeY);
+      pBikeS->FootP = pBikeS->WantedFootP;
+      
+      pBikeS->Dir = DD_RIGHT;
+      
+      bUpdateAltRider = false;
+    }
+    else if(pReplayState->cFlags & SER_BIKE_STATE_DIR_LEFT) {
+      pBikeS->Hand2P = pBikeS->WantedHand2P;
+      pBikeS->Elbow2P.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cElbowX); 
+      pBikeS->Elbow2P.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cElbowY);
+      pBikeS->Shoulder2P.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cShoulderX); 
+      pBikeS->Shoulder2P.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cShoulderY);
+      pBikeS->LowerBody2P.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cLowerBodyX); 
+      pBikeS->LowerBody2P.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cLowerBodyY);
+      pBikeS->Knee2P.x = _Map8BitsToCoord(pReplayState->fFrameX,pReplayState->fMaxXDiff,pReplayState->cKneeX); 
+      pBikeS->Knee2P.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cKneeY);
+      pBikeS->Foot2P = pBikeS->WantedFoot2P;
+      
+      pBikeS->Dir = DD_LEFT;
+      
+      bUpdateRider = false;
+    }
+    
+    Vector2f V;      
+    
+    if(bUpdateRider) {        
+      /* Calculate head position */
+      V = (pBikeS->ShoulderP - pBikeS->LowerBodyP);
+      V.normalize();
+      pBikeS->HeadP = pBikeS->ShoulderP + V*m_BikeP.fNeckLength;
+    }
+    
+    if(bUpdateAltRider) {
+      /* Calculate head position (Alt.) */
+      V = (pBikeS->Shoulder2P - pBikeS->LowerBody2P);
+      V.normalize();
+      pBikeS->Head2P = pBikeS->Shoulder2P + V*m_BikeP.fNeckLength;
+    }
+    
+    /* Internally we'd like to know the abs. relaxed position of the wheels */
+    pBikeS->RFrontWheelP.x = m_BikeA.Fp.x*pBikeS->fFrameRot[0] + m_BikeA.Fp.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->RFrontWheelP.y = m_BikeA.Fp.x*pBikeS->fFrameRot[2] + m_BikeA.Fp.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
+    pBikeS->RRearWheelP.x = m_BikeA.Rp.x*pBikeS->fFrameRot[0] + m_BikeA.Rp.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
+    pBikeS->RRearWheelP.y = m_BikeA.Rp.x*pBikeS->fFrameRot[2] + m_BikeA.Rp.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;     
+  }
+  
+#endif
+
   void MotoGame::_UpdateGameState(SerializedBikeState *pReplayState) {
     bool bUpdateRider=true,bUpdateAltRider=true;
     
@@ -452,11 +584,11 @@ namespace vapp {
       m_BikeS.FrontWheelP.y = _Map8BitsToCoord(pReplayState->fFrameY,pReplayState->fMaxYDiff,pReplayState->cFrontWheelY);
       m_BikeS.CenterP.x = pReplayState->fFrameX; 
       m_BikeS.CenterP.y = pReplayState->fFrameY;
-            
+      
       _16BitsToMatrix(pReplayState->nFrameRot,m_BikeS.fFrameRot);
       _16BitsToMatrix(pReplayState->nFrontWheelRot,m_BikeS.fFrontWheelRot);
       _16BitsToMatrix(pReplayState->nRearWheelRot,m_BikeS.fRearWheelRot);
-
+      
       /* Update engine stuff */
       m_BikeS.fBikeEngineRPM = ENGINE_MIN_RPM + (ENGINE_MAX_RPM - ENGINE_MIN_RPM) * ((float)pReplayState->cBikeEngineRPM) / 255.0f;
     }                                                                                        
@@ -576,10 +708,6 @@ namespace vapp {
       m_BikeS.Head2P = m_BikeS.Shoulder2P + V*m_BikeP.fNeckLength;
     }
 
-    /* Maybe someone want these as well */    
-    m_BikeS.pAnchors = &m_BikeA;
-    m_BikeS.pParams = &m_BikeP;
-    
     /* Internally we'd like to know the abs. relaxed position of the wheels */
     m_BikeS.RFrontWheelP.x = m_BikeA.Fp.x*m_BikeS.fFrameRot[0] + m_BikeA.Fp.y*m_BikeS.fFrameRot[1] + m_BikeS.CenterP.x;
     m_BikeS.RFrontWheelP.y = m_BikeA.Fp.x*m_BikeS.fFrameRot[2] + m_BikeA.Fp.y*m_BikeS.fFrameRot[3] + m_BikeS.CenterP.y;
@@ -839,213 +967,33 @@ namespace vapp {
   
   /*===========================================================================
   Collision detection/handling
-  ===========================================================================*/
-  int MotoGame::_AddContactToList(dContact *pContacts,int nNumContacts,dContact *pc) {
-    int i;
-    for(int i=0;i<nNumContacts;i++) {
-      if(fabs(pContacts[i].geom.pos[0] - pc->geom.pos[0]) < 0.1f &&
-         fabs(pContacts[i].geom.pos[1] - pc->geom.pos[1]) < 0.1f) return nNumContacts;
-    }
-    //printf(" [%f %f]  [%f %f]\n",pc->geom.pos[0],pc->geom.pos[1],
-      //pc->geom.normal[0],pc->geom.normal[1]);
-    memcpy(&pContacts[nNumContacts],pc,sizeof(dContact));
-    nNumContacts++;
-    return nNumContacts;
-  }
-
-  float MotoGame::_CalculateDepth(const Vector2f &Cp,float Cr,Vector2f P) {
-    float fDist = (Cp - P).length();
-    float fDepth = Cr - fDist;
-    if(fDepth<0.0f) fDepth=0.0f;
-    return fDepth;
-  }
-
-  bool MotoGame::_IntersectHeadLine(Vector2f Cp,float Cr,Vector2f A0,Vector2f A1) {
-    Vector2f V=A1-A0;
-    Vector2f En(-V.y,V.x);
-    if(En.dot(Cp) - En.dot(A0) < 0) return false; /* circle is behind, please continue */    
-    
-    /* Edge points... */
-    Vector2f w = Cp - A0;
-    float d0 = w.normalize();
-    if(d0 <= Cr) {
-      return true;
-    }
-
-    Vector2f Xw = Cp - A1;
-    float Xd0 = Xw.normalize();
-    if(Xd0 <= Cr) {
-      return true;
-    }
-
-    /* Test it */
-    Vector2f Res1,Res2;
-    int nRet = intersectLineCircle2f( Cp,Cr,A0,A1,Res1,Res2 );
-    if(nRet > 0) {
-      return true;
-    }
-    
-    return false;
-  }
-  
+  ===========================================================================*/  
   bool MotoGame::_IntersectPointLevel(Vector2f Cp) {
-  //  for(Ge    
     return false; /* TODO */
   }
   
-  bool MotoGame::_IntersectHeadLevel(Vector2f Cp,float Cr) {
-    /* For each source block in level */
-    std::vector<LevelBlock *> &BList = m_pLevelSrc->getBlockList();
+  bool MotoGame::_IntersectHeadLevel(Vector2f Cp,float Cr,const Vector2f &LastCp) {
+    if(m_Collision.checkCircle(Cp.x,Cp.y,Cr)) return true;
     
-    for(int i=0;i<BList.size();i++) {
-      if(BList[i]->bBackground) continue;
-      
-      Vector2f BlockPos = Vector2f(BList[i]->fPosX,BList[i]->fPosY);
-      
-      /* For each edge */
-      for(int j=0;j<BList[i]->Vertices.size();j++) {
-        /* Next vertex? */
-        int jnext = j+1;
-        if(jnext == BList[i]->Vertices.size()) jnext=0;        
-        
-        /* Define edge */
-        Vector2f A0(BList[i]->Vertices[j]->fX,BList[i]->Vertices[j]->fY);
-        Vector2f A1(BList[i]->Vertices[jnext]->fX,BList[i]->Vertices[jnext]->fY);
-        A0 += BlockPos; A1 += BlockPos;
-
-        /* Check it */
-        if(_IntersectHeadLine(Cp,Cr,A0,A1)) return true;
+    if(!m_bFirstPhysicsUpdate) {
+      dContact c[100];
+      int nNumContacts = m_Collision.collideLine(LastCp.x,LastCp.y,Cp.x,Cp.y,c,100);
+      if(nNumContacts > 0) {
+        return true;
       }
     }
     
-    /* Limits */
-    if(_IntersectHeadLine(Cp,Cr,
-                          Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getTopLimit()),
-                          Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getBottomLimit())))
-      return true;
-                                
-    if(_IntersectHeadLine(Cp,Cr,
-                          Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getBottomLimit()),
-                          Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getBottomLimit())))
-      return true;
-
-    if(_IntersectHeadLine(Cp,Cr,
-                          Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getBottomLimit()),
-                          Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getTopLimit())))
-      return true;
-
-    if(_IntersectHeadLine(Cp,Cr,
-                          Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getTopLimit()),
-                          Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getTopLimit())))      
-      return true;
-
     return false;
   }
 
-  int MotoGame::_IntersectWheelLine(Vector2f Cp,float Cr,int nNumContacts,dContact *pContacts,Vector2f A0,Vector2f A1) {
-    Vector2f V=A1-A0;
-    Vector2f En(-V.y,V.x);
-
-    dContact c;
-
-    if(En.dot(Cp) - En.dot(A0) < 0) return nNumContacts; /* circle is behind, please continue */    
-    
-    /* Edge points... */
-    Vector2f w = Cp - A0;
-    float d0 = w.normalize();
-    if(d0 <= Cr) {
-      /* Add contact */
-      _SetWheelContactParams(&c,A0,w,_CalculateDepth(Cp,Cr,A0));          
-      nNumContacts = _AddContactToList(pContacts,nNumContacts,&c);
-    }
-
-    Vector2f Xw = Cp - A1;
-    float Xd0 = Xw.normalize();
-    if(Xd0 <= Cr) {
-      /* Add contact */
-      _SetWheelContactParams(&c,A1,Xw,_CalculateDepth(Cp,Cr,A1));          
-      nNumContacts = _AddContactToList(pContacts,nNumContacts,&c);
-    }
-
-    /* Test it */
-    Vector2f Res1,Res2;
-    int nRet = intersectLineCircle2f( Cp,Cr,A0,A1,Res1,Res2 );
-    if(nRet > 0) {
-      /* Add contact */
-      _SetWheelContactParams(&c,Res1,Vector2f(-V.y,V.x),_CalculateDepth(Cp,Cr,Res1));
-      nNumContacts = _AddContactToList(pContacts,nNumContacts,&c);
-    }
-    if(nRet > 1) {
-      /* Add contact */
-      _SetWheelContactParams(&c,Res2,Vector2f(-V.y,V.x),_CalculateDepth(Cp,Cr,Res2));
-      nNumContacts = _AddContactToList(pContacts,nNumContacts,&c);
-    }
-    return nNumContacts;
-  }
-  
   int MotoGame::_IntersectWheelLevel(Vector2f Cp,float Cr,dContact *pContacts) {
-    int nNumContacts=0;
-    
-    /* For each source block in level */
-    std::vector<LevelBlock *> &BList = m_pLevelSrc->getBlockList();
-    
-    for(int i=0;i<BList.size();i++) {
-      if(BList[i]->bBackground) continue;
-      
-      Vector2f BlockPos = Vector2f(BList[i]->fPosX,BList[i]->fPosY);
-      
-      /* For each edge */
-      for(int j=0;j<BList[i]->Vertices.size();j++) {
-        /* Next vertex? */
-        int jnext = j+1;
-        if(jnext == BList[i]->Vertices.size()) jnext=0;        
-        
-        /* Define edge */
-        Vector2f A0(BList[i]->Vertices[j]->fX,BList[i]->Vertices[j]->fY);
-        Vector2f A1(BList[i]->Vertices[jnext]->fX,BList[i]->Vertices[jnext]->fY);
-        A0 += BlockPos; A1 += BlockPos;
-
-        nNumContacts = _IntersectWheelLine(Cp,Cr,nNumContacts,pContacts,A0,A1);
-      }
+    int nNumContacts = m_Collision.collideCircle(Cp.x,Cp.y,Cr,pContacts,100);
+    if(nNumContacts == 0) {
+      /* Nothing... but what if we are moving so fast that the circle has moved
+         all the way through some geometry? Check it's path. */
+      //nNumContacts = m_Collision.collideLine(
     }
-    
-    /* Level limits */
-    nNumContacts = _IntersectWheelLine(Cp,Cr,nNumContacts,pContacts,
-                                       Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getTopLimit()),
-                                       Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getBottomLimit()));
-    nNumContacts = _IntersectWheelLine(Cp,Cr,nNumContacts,pContacts,
-                                       Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getBottomLimit()),
-                                       Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getBottomLimit()));
-    nNumContacts = _IntersectWheelLine(Cp,Cr,nNumContacts,pContacts,
-                                       Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getBottomLimit()),
-                                       Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getTopLimit()));
-    nNumContacts = _IntersectWheelLine(Cp,Cr,nNumContacts,pContacts,
-                                       Vector2f(m_pLevelSrc->getRightLimit(),m_pLevelSrc->getTopLimit()),
-                                       Vector2f(m_pLevelSrc->getLeftLimit(),m_pLevelSrc->getTopLimit()));
-    
-    /* Return */
     return nNumContacts;
   }
   
-  /*===========================================================================
-  Fill a dContact structure so it functions as a wheel contact
-  ===========================================================================*/
-  void MotoGame::_SetWheelContactParams(dContact *pc,const Vector2f &Pos,const Vector2f &NormalT,float fDepth) {
-    memset(pc,0,sizeof(dContact));
-    Vector2f Normal = NormalT;
-    Normal.normalize();
-    pc->geom.depth = fDepth;
-    pc->geom.normal[0] = Normal.x;
-    pc->geom.normal[1] = Normal.y;
-    pc->geom.pos[0] = Pos.x;
-    pc->geom.pos[1] = Pos.y;
-    //pc->surface.mu = dInfinity;
-    pc->surface.mu = PHYS_WHEEL_GRIP;
-    //pc->surface.mu = 0.9; //.7; //8;
-    //pc->surface.bounce = 0.3;
-    pc->surface.soft_erp = PHYS_WHEEL_ERP;     
-    pc->surface.soft_cfm = PHYS_WHEEL_CFM; 
-    pc->surface.mode = dContactApprox1_1 | dContactSlip1;
-  }  
-  
 };
