@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -28,12 +28,15 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "LevelSrc.h"
 #include "BSP.h"
 #include "DBuffer.h"
+#include "Collision.h"
 
 #define GAME_EVENT_QUEUE_SIZE				128
 #define GAME_EVENT_OUTGOING_BUFFER  65536
 
 namespace vapp {
 
+  class Replay;
+
   /*===========================================================================
   Defines
   ===========================================================================*/
@@ -285,7 +288,6 @@ namespace vapp {
     Vector2f PrevPHq2;    /* Previous error (player hand) (Alt.) */
     
     /* Bonusinfo */
-    BikeParams *pParams;
     BikeAnchors *pAnchors;
     
     /* Driving */
@@ -302,7 +304,8 @@ namespace vapp {
     ET_ENDOFLEVEL,
     ET_WRECKER,
     ET_STRAWBERRY,
-    ET_PARTICLESOURCE
+    ET_PARTICLESOURCE,
+    ET_DUMMY
   };
 
 	/*===========================================================================
@@ -354,6 +357,16 @@ namespace vapp {
     /* ET_PARTICLESOURCE */
     std::string ParticleType;
     float fNextParticleTime;
+    
+    /* ET_DUMMY */
+  };
+
+	/*===========================================================================
+	Requested player state
+  ===========================================================================*/
+  struct GameReqPlayerPos {
+    Vector2f Pos;
+    bool bDriveRight;  
   };
 
 	/*===========================================================================
@@ -365,17 +378,30 @@ namespace vapp {
     float fRemoveTime;                /* The time when it should be removed */
     std::string Text;                 /* The text */
     int nAlpha;                       /* Alpha amount */
+    bool bOnce;                       /* Unique message */
   };  
 
 	/*===========================================================================
 	Game event types
   ===========================================================================*/
   enum GameEventType {
-		GAME_EVENT_PLAYER_DIES,
-		GAME_EVENT_PLAYER_ENTERS_ZONE,
-		GAME_EVENT_PLAYER_LEAVES_ZONE,
-		GAME_EVENT_PLAYER_TOUCHES_ENTITY,
-		GAME_EVENT_ENTITY_DESTROYED,
+    GAME_EVENT_PLAYER_DIES,
+    GAME_EVENT_PLAYER_ENTERS_ZONE,
+    GAME_EVENT_PLAYER_LEAVES_ZONE,
+    GAME_EVENT_PLAYER_TOUCHES_ENTITY,
+    GAME_EVENT_ENTITY_DESTROYED,
+    
+    GAME_EVENT_LUA_CALL_CLEARMESSAGES,
+    GAME_EVENT_LUA_CALL_PLACEINGAMEARROW,
+    GAME_EVENT_LUA_CALL_PLACESCREENARROW,
+    GAME_EVENT_LUA_CALL_HIDEARROW,
+    GAME_EVENT_LUA_CALL_MESSAGE,
+    GAME_EVENT_LUA_CALL_MOVEBLOCK,
+    GAME_EVENT_LUA_CALL_SETBLOCKPOS,
+    GAME_EVENT_LUA_CALL_SETGRAVITY,
+    GAME_EVENT_LUA_CALL_SETPLAYERPOSITION,
+    GAME_EVENT_LUA_CALL_SETENTITYPOS
+
   };
   
 	/*===========================================================================
@@ -405,69 +431,155 @@ namespace vapp {
 		char cEntityID[64];					  /* ID of entity */
 		bool bHead;										/* Touched with head? */
   };
+
+
+  struct GameEventLuaCallClearmessages {
+    /* nothing */
+  };
+  
+  struct GameEventLuaCallPlaceingamearrow {
+    float x, y, angle;
+  };
+  
+  struct GameEventLuaCallPlacescreenarrow {
+    float x, y, angle;
+  };
+  
+  struct GameEventLuaCallHidearrow {
+  };
+  
+  struct GameEventLuaCallMessage {
+    char cMessage[512]; /* message */
+  };
+  
+  struct GameEventLuaCallMoveblock {
+    char cBlockID[64]; /* ID of block */
+    float x, y;
+  };
+  
+  struct GameEventLuaCallSetblockpos {
+    char cBlockID[64]; /* ID of block */
+    float x, y;
+  };
+  
+  struct GameEventLuaCallSetgravity {
+    float x, y;
+  };
+  
+  struct GameEventLuaCallSetplayerposition {
+    float x, y;
+    bool bRight;
+  };
+  
+  struct GameEventLuaCallSetentitypos {
+    char cEntityID[64]; /* ID of entity */
+    float x,y;
+  };
   
   struct GameEvent {
     int nSeq;                         /* Sequence number */
-		GameEventType Type;								/* Type of event */
-		
-		union GameEvent_u {								
-			/* GAME_EVENT_PLAYER_DIES */		
-			GameEventPlayerDies PlayerDies;								
-			
-			/* GAME_EVENT_ENTITY_DESTROYED */
-			GameEventEntityDestroyed EntityDestroyed;
-			
-			/* GAME_EVENT_PLAYER_ENTERS_ZONE */
-			GameEventPlayerEntersZone PlayerEntersZone;
-			
-			/* GAME_EVENT_PLAYER_LEAVES_ZONE */
-			GameEventPlayerLeavesZone PlayerLeavesZone;
+    GameEventType Type;								/* Type of event */
+    
+    union GameEvent_u {								
+      /* GAME_EVENT_PLAYER_DIES */		
+      GameEventPlayerDies PlayerDies;								
+      
+      /* GAME_EVENT_ENTITY_DESTROYED */
+      GameEventEntityDestroyed EntityDestroyed;
+      
+      /* GAME_EVENT_PLAYER_ENTERS_ZONE */
+      GameEventPlayerEntersZone PlayerEntersZone;
+      
+      /* GAME_EVENT_PLAYER_LEAVES_ZONE */
+      GameEventPlayerLeavesZone PlayerLeavesZone;
+      
+      /* GAME_EVENT_PLAYER_TOUCHES_ENTITY */
+      GameEventPlayerTouchesEntity PlayerTouchesEntity;
 
-			/* GAME_EVENT_PLAYER_TOUCHES_ENTITY */
-			GameEventPlayerTouchesEntity PlayerTouchesEntity;
-		} u;		
+
+      /* GAME_EVENT_LUA_CALL_CLEARMESSAGES     */
+      GameEventLuaCallClearmessages LuaCallClearmessages;
+
+      /* GAME_EVENT_LUA_CALL_PLACEINGAMEARROW  */
+      GameEventLuaCallPlaceingamearrow LuaCallPlaceingamearrow;
+
+      /* GAME_EVENT_LUA_CALL_PLACESCREENARROW  */
+      GameEventLuaCallPlacescreenarrow LuaCallPlacescreenarrow;
+
+      /* GAME_EVENT_LUA_CALL_HIDEARROW	 */
+      GameEventLuaCallHidearrow LuaCallHidearrow;
+
+      /* GAME_EVENT_LUA_CALL_MESSAGE		 */
+      GameEventLuaCallMessage LuaCallMessage;
+
+      /* GAME_EVENT_LUA_CALL_MOVEBLOCK	 */
+      GameEventLuaCallMoveblock LuaCallMoveblock;
+
+      /* GAME_EVENT_LUA_CALL_SETBLOCKPOS	 */
+      GameEventLuaCallSetblockpos LuaCallSetblockpos;
+
+      /* GAME_EVENT_LUA_CALL_SETGRAVITY	 */
+      GameEventLuaCallSetgravity LuaCallSetgravity;
+
+      /* GAME_EVENT_LUA_CALL_SETPLAYERPOSITION */
+      GameEventLuaCallSetplayerposition LuaCallSetplayerposition;
+
+      /* GAME_EVENT_LUA_CALL_SETENTITYPOS      */
+      GameEventLuaCallSetentitypos LuaCallSetentitypos;
+
+    } u;		
   };
-  
+    
   struct RecordedGameEvent {
     float fTime;                    /* Time of event */
     GameEvent Event;                /* Event itself */
     bool bPassed;                   /* Whether we have passed it */
   };
-    
+
 	/*===========================================================================
 	Game object
   ===========================================================================*/
   class MotoGame {
     public:          
       MotoGame() {m_pLevelSrc=NULL;
-                  clearStates();}
+                  clearStates();
+		  m_lastCallToEveryHundreath = 0.0;
+#if defined(ALLOW_GHOST)
+		  m_showGhostTimeDiff = true;
+#endif
+      }
       ~MotoGame() {endLevel();}     
     
       /* Methods */
-      void playLevel(LevelSrc *pLevelSrc);
-      void updateLevel(float fTimeStep,SerializedBikeState *pReplayState,DBuffer *pEventReplayBuffer);
+      void playLevel(LevelSrc *pLevelSrc, bool bIsAReplay);
+      void updateLevel(float fTimeStep,SerializedBikeState *pReplayState,Replay *p_replay);
       void endLevel(void);
       
-      void touchEntity(Entity *pEntity,bool bHead); 
+      void touchEntity(Entity *pEntity,bool bHead, bool bEnableScript); 
       void deleteEntity(Entity *pEntity);
       int countEntitiesByType(EntityType Type);
       Entity *findEntity(const std::string &ID);
       
       void clearStates(void);
       
-      void gameMessage(std::string Text);
+      void gameMessage(std::string Text,bool bOnce = false);
       void clearGameMessages(void);
       
       void getSerializedBikeState(SerializedBikeState *pState);
-      void unserializeGameEvents(DBuffer &Buffer);
+      static void unserializeGameEvents(DBuffer *Buffer, std::vector<RecordedGameEvent *> *v_ReplayEvents);
       void interpolateGameState(SerializedBikeState *pA,SerializedBikeState *pB,SerializedBikeState *p,float t);
 
       float getBikeEngineRPM(void);
-      
+      float getBikeEngineSpeed();
+
       GameEvent *createGameEvent(GameEventType Type);
       GameEvent *getNextGameEvent(void);
       int getNumPendingGameEvents(void);
       
+      void setPlayerPosition(float x,float y,bool bFaceRight);
+      const Vector2f &getPlayerPosition(void);
+      bool getPlayerFaceDir(void);
+      
       /* Direct Lua interaction methods */
       bool scriptCallBool(std::string FuncName,bool bDefault=false);
       void scriptCallVoid(std::string FuncName);
@@ -479,6 +591,18 @@ namespace vapp {
       LevelSrc *getLevelSrc(void) {return m_pLevelSrc;}
       std::vector<ConvexBlock *> &getBlocks(void) {return m_Blocks;}
       BikeState *getBikeState(void) {return &m_BikeS;}
+
+      BikeParams *getBikeParams() { return &m_BikeP;}
+
+#if defined(ALLOW_GHOST)
+      BikeState *getGhostBikeState(void) {return &m_GhostBikeS;}
+      bool isGhostActive() {return m_isGhostActive;}
+      void setGhostActive(bool s) {m_isGhostActive = s;}
+      std::vector<float> m_myLastStrawberry;
+      std::vector<float> m_ghostLastStrawberry;
+      float m_myDiffOfGhost; /* time diff between the ghost and the player */
+      int m_myDiffOfGhostLastStrawberry; /* last Strawberry for which the calcul has been done */
+#endif
       BikeController *getBikeController(void) {return &m_BikeC;}
       std::vector<GameMessage *> &getGameMessage(void) {return m_GameMessages;}
       std::vector<Entity *> &getEntities(void) {return m_Entities;}
@@ -492,13 +616,22 @@ namespace vapp {
       bool isWheelSpinning(void) {return m_bWheelSpin;}
       Vector2f getWheelSpinPoint(void) {return m_WheelSpinPoint;}
       Vector2f getWheelSpinDir(void) {return m_WheelSpinDir;}
-            
+      CollisionSystem *getCollisionHandler(void) {return &m_Collision;}
+      void setGravity(float x,float y) {m_PhysGravity.x=x; m_PhysGravity.y=y;}
+      const Vector2f &getGravity(void) {return m_PhysGravity;}
+        
       /* Debug */
       void resetDummies(void) {m_nNumDummies=0;}
       int getNumDummies(void) {return m_nNumDummies;}
       DummyHelper *getDummies(void) {return m_Dummies;}
       void addDummy(Vector2f Pos,float r,float g,float b);
     
+#if defined(ALLOW_GHOST)  
+      void UpdateGhostFromReplay(Replay *p_replay, SerializedBikeState *pReplayState);
+      float getGhostDiff() {return m_myDiffOfGhost;}
+      void setShowGhostTimeDiff(bool b) { m_showGhostTimeDiff = b; }
+#endif
+
     private:         
       /* Data */
       int m_nGameEventQueueReadIdx,m_nGameEventQueueWriteIdx;
@@ -512,8 +645,12 @@ namespace vapp {
       
       int m_nStillFrames;
 
+      Vector2f m_PhysGravity; /* gravity */
+
       ArrowPointer m_Arrow;               /* Arrow */  
       
+      CollisionSystem m_Collision;        /* Collision system */
+            
       LevelSrc *m_pLevelSrc;              /* Source of level */            
       lua_State *m_pL;                    /* Lua state associated with the
                                              level */
@@ -521,23 +658,35 @@ namespace vapp {
       std::vector<Entity *> m_Entities;   /* Entities */
       dWorldID m_WorldID;                 /* World ID */
       
-      std::vector<RecordedGameEvent *> m_ReplayEvents; /* Events reconstructed from replay */
-      
       std::vector<Entity *> m_FSprites;   /* Foreground sprites */
       std::vector<Entity *> m_BSprites;   /* Background sprites */
       std::vector<OverlayEdge *> m_OvEdges;/* Overlay edges */
       std::vector<Entity *> m_DelSchedule;/* Entities scheduled for deletion */
       std::vector<GameMessage *> m_GameMessages;
       
-      BikeParams m_BikeP;                 /* Bike parameters */      
+      BikeParams m_BikeP;                 /* Bike physics */      
       BikeAnchors m_BikeA;                /* Important bike anchor points */
       BikeState m_BikeS;                  /* Bike state */
+
+#if defined(ALLOW_GHOST)  
+      BikeState m_GhostBikeS;             /* ghost state */
+      bool m_isGhostActive;               /* is ghost active : must it be displayed, ... */
+      bool m_showGhostTimeDiff;
+#endif
+
       BikeController m_BikeC;             /* Bike controller */
       
       bool m_bFinished,m_bDead;           /* Yir */
       
       int m_nLastEventSeq;
       
+      bool m_bFirstPhysicsUpdate;
+
+      Vector2f m_PrevRearWheelP;          /* Prev. rear wheel position */
+      Vector2f m_PrevFrontWheelP;         /* Prev. front wheel position */
+      Vector2f m_PrevHeadP;
+      Vector2f m_PrevHead2P;
+      
       /* Wheels spinning dirt up... muzakka! :D */
       bool m_bWheelSpin;                  /* Do it captain */
       Vector2f m_WheelSpinPoint,m_WheelSpinDir; /* Where and how much */
@@ -587,8 +736,15 @@ namespace vapp {
       dJointID m_ElbowHingeID2;
       dJointID m_HandHingeID2;
             
-      dJointGroupID m_ContactGroup;       /* Contact joint group */      
-                                             
+      dJointGroupID m_ContactGroup;       /* Contact joint group */     
+      
+      /* Teleport next frame? */
+      bool m_bTeleport;
+      GameReqPlayerPos m_TeleportDest; 
+            
+      /* for EveryHundreath function */
+      float m_lastCallToEveryHundreath;
+                
       /* Helpers */
       void _GenerateLevel(void);          /* Called by playLevel() to 
                                              prepare the level */
@@ -596,11 +752,8 @@ namespace vapp {
       void _CalculateBikeAnchors(void);
       int _IntersectWheelLevel(Vector2f Cp,float Cr,dContact *pContacts);
       int _IntersectWheelLine(Vector2f Cp,float Cr,int nNumContacts,dContact *pContacts,Vector2f A0,Vector2f A1);
-      bool _IntersectHeadLevel(Vector2f Cp,float Cr);
+      bool _IntersectHeadLevel(Vector2f Cp,float Cr,const Vector2f &LastCp);
       bool _IntersectHeadLine(Vector2f Cp,float Cr,Vector2f A0,Vector2f A1);
-      void _SetWheelContactParams(dContact *pc,const Vector2f &Pos,const Vector2f &Normal,float fDepth);
-      int _AddContactToList(dContact *pContacts,int nNumContacts,dContact *pc);      
-      float _CalculateDepth(const Vector2f &Cp,float Cr,Vector2f P);
       bool _DoCircleTouchZone(const Vector2f &Cp,float Cr,LevelZone *pZone);
       bool _IntersectPointLevel(Vector2f Cp);
       void _UpdateZones(void);
@@ -610,13 +763,17 @@ namespace vapp {
       EdgeEffect _TransEdgeEffect(std::string Name);
       void _UpdateEntities(void);
       void _UpdateGameState(SerializedBikeState *pReplayState);
+#if defined(ALLOW_GHOST)
+      /* static */ void _UpdateStateFromReplay(SerializedBikeState *pReplayState,BikeState *pBikeS);
+      void UpdateDiffFromGhost();
+#endif
       char _MapCoordTo8Bits(float fRef,float fMaxDiff,float fCoord);
       float _Map8BitsToCoord(float fRef,float fMaxDiff,char c);
       unsigned short _MatrixTo16Bits(const float *pfMatrix);
       void _16BitsToMatrix(unsigned short n16,float *pfMatrix);
       void _SerializeGameEventQueue(DBuffer &Buffer,GameEvent *pEvent);
       
-      void _UpdateReplayEvents(void);
+      void _UpdateReplayEvents(Replay *p_replay);
       void _HandleReplayEvent(GameEvent *pEvent);
       void _HandleReverseReplayEvent(GameEvent *pEvent);
       
@@ -626,7 +783,13 @@ namespace vapp {
       void _UninitPhysics(void);
       void _PrepareBikePhysics(Vector2f StartPos);
       void _PrepareRider(Vector2f StartPos);
-            
+         
+      /* action for events */
+      void _SetEntityPos(String pEntityID, float pX, float pY);
+      void _PlaceInGameArrow(float pX, float pY, float pAngle);
+      void _PlaceScreenArrow(float pX, float pY, float pAngle);
+      void _MoveBlock(String pBlockID, float pX, float pY);
+      void _SetBlockPos(String pBlockID, float pX, float pY);
     };
 
 };
