@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -26,6 +26,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VFileIO.h"
 #include "MotoGame.h"
 #include "Renderer.h"
+#include "GameText.h"
 
 namespace vapp {
   
@@ -50,6 +51,26 @@ namespace vapp {
     m_fCurrentHorizontalScrollShift = 0.0f;
     m_fDesiredHorizontalScrollShift = 0.0f;
     m_fNextParticleUpdate = 0.0f;
+    
+    #if defined(ALLOW_GHOST)
+      /* Set initial ghost information position */
+      if(getGameObject()->isGhostActive()) {
+        m_GhostInfoPos = getGameObject()->getGhostBikeState()->CenterP + Vector2f(0,-1.5f);
+        m_GhostInfoVel = Vector2f(0,0);
+
+        m_fNextGhostInfoUpdate = 0.0f;
+        m_nGhostInfoTrans = 255;
+        
+        if(m_pGhostReplay != NULL) {
+          m_GhostInfoString = std::string(GAMETEXT_GHOSTOF) + m_pGhostReplay->getPlayerName() +
+                              std::string("\n(") + m_ReplayDesc + std::string(")") +
+                              std::string("\n(") + getParent()->formatTime(m_pGhostReplay->getFinishTime()) + std::string(")");
+        }
+        
+        if(m_ReplayDesc == "")
+          m_nGhostInfoTrans = 0;
+      }
+    #endif
         
     /* Optimize scene */
     std::vector<ConvexBlock *> &Blocks = getGameObject()->getBlocks();
@@ -66,6 +87,10 @@ namespace vapp {
         Center = Vector2f(Blocks[i]->pSrcBlock->fPosX,Blocks[i]->pSrcBlock->fPosY);
         pTexture = getParent()->TexMan.getTexture(Blocks[i]->pSrcBlock->Texture);
         if(pTexture != NULL) {GLName = pTexture->nID;}
+        else {
+          Log("** Warning ** : Texture '%s' not found!",Blocks[i]->pSrcBlock->Texture.c_str());
+          getGameObject()->gameMessage(GAMETEXT_MISSINGTEXTURES,true);          
+        }
       }
       else {
         pTexture = getParent()->TexMan.getTexture("default");
@@ -191,6 +216,7 @@ namespace vapp {
                          MAKE_COLOR(0,0,0,MINIMAPALPHA),MAKE_COLOR(255,255,255,MINIMAPALPHA));
     getParent()->scissorGraphics(x+1,y+1,nWidth-2,nHeight-2);
     glEnable(GL_SCISSOR_TEST);
+    glLoadIdentity();
         
     MotoGame *pGame = getGameObject();
     std::vector<ConvexBlock *> &Blocks = pGame->getBlocks();
@@ -207,6 +233,7 @@ namespace vapp {
       glColor3f(0.5,0.5,0.5);
       for(int j=0;j<Blocks[i]->Vertices.size();j++) {
         Vector2f P = Center + Blocks[i]->Vertices[j]->P;
+        
         MINIVERTEX(P.x,P.y);                  
       }
       glEnd();
@@ -235,10 +262,21 @@ namespace vapp {
     getParent()->scissorGraphics(0,0,getParent()->getDispWidth(),getParent()->getDispHeight());
   }
 
+  void GameRenderer::zoom(float p_f) {
+    m_fScale += p_f;
+    if(m_fScale < 0) {
+      m_fScale = 0;
+    }
+  }
+
+  void GameRenderer::initZoom() {
+    m_fScale = ZOOM_DEFAULT;
+  }
+
   /*===========================================================================
   Main rendering function
   ===========================================================================*/
-  void GameRenderer::render(void) {  
+  void GameRenderer::render(void) {
     /* Update time */    
     m_pInGameStats->showWindow(true);
     m_pPlayTime->setCaption(getParent()->formatTime(getGameObject()->getTime()));
@@ -277,13 +315,15 @@ namespace vapp {
 
     m_Scroll += Vector2f(m_fCurrentHorizontalScrollShift,0.0f);
 
+    glLoadIdentity();
+
     /* SKY! */
     if(!m_bUglyMode)
 			_RenderSky();
 
     /* Perform scaling/translation */
-    float fScale = 0.195f;
-    glScalef(fScale * ((float)getParent()->getDispHeight()) / getParent()->getDispWidth(),fScale,1);
+    
+    glScalef(m_fScale * ((float)getParent()->getDispHeight()) / getParent()->getDispWidth(), m_fScale,1);
     //glRotatef(getGameObject()->getTime()*100,0,0,1); /* Uncomment this line if you want to vomit :) */
     glTranslatef(m_Scroll.x,m_Scroll.y,0);
     
@@ -310,9 +350,42 @@ namespace vapp {
     /* ... the entities ... */
     _RenderEntities();
 
+#if defined(ALLOW_GHOST)
+    if(getGameObject()->isGhostActive()) {
+      /* ... followed by the ghost ... Render into overlay? */
+      if(m_bGhostMotionBlur && getParent()->useFBOs()) {
+        m_Overlay.beginRendering();
+        m_Overlay.fade(0.15);
+      }
+      _RenderBike(getGameObject()->getGhostBikeState(), getGameObject()->getBikeParams(), &theme_ghost);
+      
+      if(m_bGhostMotionBlur && getParent()->useFBOs()) {
+        GLuint nOverlayTextureID = m_Overlay.endRendering();
+        m_Overlay.present();
+      }
+      
+      if(m_nGhostInfoTrans > 0) {
+        _RenderInGameText(m_GhostInfoPos,m_GhostInfoString,MAKE_COLOR(255,255,255,m_nGhostInfoTrans));
+        if(getGameObject()->getTime() > m_fNextGhostInfoUpdate) {
+          if(getGameObject()->getTime() - m_fNextGhostInfoUpdate > 0.05f) {
+            if(getGameObject()->getTime() > 3.0f) m_nGhostInfoTrans=0;
+            
+            m_GhostInfoPos = getGameObject()->getGhostBikeState()->CenterP + Vector2f(0,-1.5);
+          }
+          else {
+            if(getGameObject()->getTime() > 3.0f) m_nGhostInfoTrans-=16;
+            
+            m_GhostInfoVel = ((getGameObject()->getGhostBikeState()->CenterP + Vector2f(0,-1.5)) - m_GhostInfoPos) * 0.2f;
+            m_GhostInfoPos += m_GhostInfoVel;
+          }
+          m_fNextGhostInfoUpdate = getGameObject()->getTime() + 0.025f;        
+        }
+      }      
+    }
+#endif
+
     /* ... followed by the bike ... */
-    _RenderBike();
-    
+    _RenderBike(getGameObject()->getBikeState(), getGameObject()->getBikeParams(), &theme_normal);
     if(m_Quality == GQ_HIGH && !m_bUglyMode) {
       /* Render particles (front!) */    
       _RenderParticles();
@@ -322,6 +395,79 @@ namespace vapp {
       /* ... and finally the foreground sprites! */
       _RenderSprites(true,false);
     }
+    
+    //glBegin(GL_LINE_STRIP);
+    //glColor3f(1,1,1);
+    //glVertex2f(getGameObject()->m_PrevFrontWheelP.x,getGameObject()->m_PrevFrontWheelP.y);
+    //glVertex2f(getGameObject()->getBikeState()->FrontWheelP.x,getGameObject()->getBikeState()->FrontWheelP.y);
+    //glEnd();
+    //    
+    //glBegin(GL_LINE_STRIP);
+    //glColor3f(1,1,1);
+    //glVertex2f(getGameObject()->m_PrevRearWheelP.x,getGameObject()->m_PrevRearWheelP.y);
+    //glVertex2f(getGameObject()->getBikeState()->RearWheelP.x,getGameObject()->getBikeState()->RearWheelP.y);
+    //glEnd();
+        
+    if(isDebug()) {
+      /* Draw some collision handling debug info */
+      CollisionSystem *pc = getGameObject()->getCollisionHandler();
+      for(int i=0;i<pc->m_CheckedLines.size();i++) {
+        glLineWidth(3);
+ 			  glBegin(GL_LINE_STRIP);
+  		  glColor3f(1,0,0);
+  		  glVertex2f(pc->m_CheckedLines[i]->x1,pc->m_CheckedLines[i]->y1);
+  		  glVertex2f(pc->m_CheckedLines[i]->x2,pc->m_CheckedLines[i]->y2);
+  		  glEnd();
+        glLineWidth(2);
+      }
+      for(int i=0;i<pc->m_CheckedCells.size();i++) {
+        glBegin(GL_LINE_LOOP);
+        glColor3f(1,0,0);
+        glVertex2f(pc->m_CheckedCells[i].x1,pc->m_CheckedCells[i].y1);
+        glVertex2f(pc->m_CheckedCells[i].x2,pc->m_CheckedCells[i].y1);
+        glVertex2f(pc->m_CheckedCells[i].x2,pc->m_CheckedCells[i].y2);
+        glVertex2f(pc->m_CheckedCells[i].x1,pc->m_CheckedCells[i].y2);
+        glEnd();
+      }
+      for(int i=0;i<pc->m_CheckedLinesW.size();i++) {
+        glLineWidth(1);
+ 			  glBegin(GL_LINE_STRIP);
+  		  glColor3f(0,1,0);
+  		  glVertex2f(pc->m_CheckedLinesW[i]->x1,pc->m_CheckedLinesW[i]->y1);
+  		  glVertex2f(pc->m_CheckedLinesW[i]->x2,pc->m_CheckedLinesW[i]->y2);
+  		  glEnd();
+        glLineWidth(1);
+      }
+      for(int i=0;i<pc->m_CheckedCellsW.size();i++) {
+        glBegin(GL_LINE_LOOP);
+        glColor3f(0,1,0);
+        glVertex2f(pc->m_CheckedCellsW[i].x1,pc->m_CheckedCellsW[i].y1);
+        glVertex2f(pc->m_CheckedCellsW[i].x2,pc->m_CheckedCellsW[i].y1);
+        glVertex2f(pc->m_CheckedCellsW[i].x2,pc->m_CheckedCellsW[i].y2);
+        glVertex2f(pc->m_CheckedCellsW[i].x1,pc->m_CheckedCellsW[i].y2);
+        glEnd();
+      }
+    }
+    
+    //const std::vector<LineSoup *> &LSoups = getGameObject()->getCollisionHandler()->getSoups();
+    //for(int i=0;i<LSoups.size();i++) {
+    //  glLineWidth(3);
+    //  for(int j=0;j<LSoups[i]->cNumLines;j++) {
+  		//	glBegin(GL_LINE_STRIP);
+	  	//	glColor3f(LSoups[i]->r,LSoups[i]->g,LSoups[i]->b);
+	  	//	glVertex2f(LSoups[i]->Lines[j].x1,LSoups[i]->Lines[j].y1);
+	  	//	glVertex2f(LSoups[i]->Lines[j].x2,LSoups[i]->Lines[j].y2);
+	  	//	glEnd();
+    //  }
+    //  glLineWidth(1);
+    //  glBegin(GL_LINE_LOOP);
+  		//glColor3f(LSoups[i]->r,LSoups[i]->g,LSoups[i]->b);
+  		//glVertex2f(LSoups[i]->Min.x,LSoups[i]->Min.y);
+  		//glVertex2f(LSoups[i]->Max.x,LSoups[i]->Min.y);
+  		//glVertex2f(LSoups[i]->Max.x,LSoups[i]->Max.y);
+  		//glVertex2f(LSoups[i]->Min.x,LSoups[i]->Max.y);
+    //  glEnd();
+    //}
         
     /* Hmm, in debug-mode we'd also like to see the zones and stuff */
     if(isDebug()) {    
@@ -373,6 +519,63 @@ namespace vapp {
 
     /* And then the game messages */
     _RenderGameMessages();            
+    
+    /* If there's strawberries in the level, tell the user how many there's left */
+    _RenderGameStatus();
+  }
+
+  /*===========================================================================
+  Game status rendering
+  ===========================================================================*/
+  void GameRenderer::_RenderGameStatus(void) {
+    MotoGame *pGame = getGameObject();
+
+    int nStrawberriesLeft = pGame->countEntitiesByType(ET_STRAWBERRY);
+    Texture *pIcon = m_pFlowerAnim->m_Frames[0]->pTexture;
+    int nQuantity = 0;
+    
+    if(nStrawberriesLeft > 0) {
+      pIcon = m_pStrawberryAnim->m_Frames[0]->pTexture;
+      nQuantity = nStrawberriesLeft;
+    }
+            
+    float x1 = 90;
+    float y1 = -2;
+    float x2 = 115;
+    float y2 = 23;
+    
+    glEnable(GL_BLEND); 
+    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
+    glBindTexture(GL_TEXTURE_2D,pIcon->nID);
+    glEnable(GL_TEXTURE_2D);
+    glBegin(GL_POLYGON);
+    glColor3f(1,1,1);
+    glTexCoord2f(0,0);
+    getParent()->glVertex(x1,y1);
+    glTexCoord2f(1,0);
+    getParent()->glVertex(x2,y1);
+    glTexCoord2f(1,1);
+    getParent()->glVertex(x2,y2);
+    glTexCoord2f(0,1);
+    getParent()->glVertex(x1,y2);
+    glEnd();
+    glDisable(GL_TEXTURE_2D);
+    glDisable(GL_BLEND);        
+
+    if(nQuantity > 0) {
+      int tx1,ty1,tx2,ty2;
+      char cBuf[256];    
+      sprintf(cBuf,"%d",nQuantity);
+      UITextDraw::getTextExt(getSmallFont(),cBuf,&tx1,&ty1,&tx2,&ty2);
+      
+      /* Now for some evil special-case adjustments */
+      int nAdjust = 0;
+      if(nQuantity == 1) nAdjust = -3;
+      if(nQuantity > 9) nAdjust = -2;
+      
+      /* Draw text */
+      UITextDraw::printRaw(getSmallFont(),nAdjust + (x1+x2)/2 - (tx2-tx1)/2,y2-(ty2-ty1)+3,cBuf,MAKE_COLOR(255,255,0,255));
+    }
   }
   
   /*===========================================================================
@@ -390,7 +593,8 @@ namespace vapp {
                   getParent()->getDispHeight()/2 - (float)(pArrow->ArrowPointerPos.y + m_Scroll.y)*m_fZoom);      
       }
       else if(pArrow->nArrowPointerMode == 2) {          
-        C=pArrow->ArrowPointerPos;
+        C.x=(getParent()->getDispWidth() * pArrow->ArrowPointerPos.x) / 800.0f;
+        C.y=(getParent()->getDispHeight() * pArrow->ArrowPointerPos.y) / 600.0f;
       }
       Vector2f p1,p2,p3,p4;
       p1 = Vector2f(1,0); p1.rotateXY(pArrow->fArrowPointerAngle);
@@ -475,6 +679,8 @@ namespace vapp {
            Vector2f(-pType->Center.x,-pType->Center.y);
             
       glEnable(GL_BLEND); 
+      glEnable(GL_ALPHA_TEST);
+      glAlphaFunc(GL_GEQUAL,0.5f);
       glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
       glBindTexture(GL_TEXTURE_2D,GLName);
       glEnable(GL_TEXTURE_2D);
@@ -490,6 +696,7 @@ namespace vapp {
       _Vertex(p3);
       glEnd();
       glDisable(GL_TEXTURE_2D);
+      glDisable(GL_ALPHA_TEST);
       glDisable(GL_BLEND);
 
       /* Debug mode? */
@@ -537,12 +744,25 @@ namespace vapp {
 					glEnd();
 				}
 			}
+			
+			/* Also render the level limits */
+			glBegin(GL_LINE_LOOP);
+			glColor3f(1,1,1);
+			glVertex2f(getGameObject()->getLevelSrc()->getLeftLimit(),getGameObject()->getLevelSrc()->getTopLimit());
+			glVertex2f(getGameObject()->getLevelSrc()->getRightLimit(),getGameObject()->getLevelSrc()->getTopLimit());
+			glVertex2f(getGameObject()->getLevelSrc()->getRightLimit(),getGameObject()->getLevelSrc()->getBottomLimit());
+			glVertex2f(getGameObject()->getLevelSrc()->getLeftLimit(),getGameObject()->getLevelSrc()->getBottomLimit());
+			glEnd();
 		}
 		else {
 			/* Render all non-background blocks */
 			/* Static geoms... */
 			for(int i=0;i<m_Geoms.size();i++) {
-				glBindTexture(GL_TEXTURE_2D,m_Geoms[i]->pTexture->nID);
+			  if(m_Geoms[i]->pTexture != NULL)
+				  glBindTexture(GL_TEXTURE_2D,m_Geoms[i]->pTexture->nID);				  
+				else
+				  glBindTexture(GL_TEXTURE_2D,0);	/* no texture */
+				  
 				glEnable(GL_TEXTURE_2D);      
 				glColor3f(1,1,1);
 	      
@@ -846,7 +1066,7 @@ namespace vapp {
   /*===========================================================================
   Free stuff
   ===========================================================================*/
-  void GameRenderer::_Free(void) {
+  void GameRenderer::_Free(void) {    
     /* Free animations */
     for(int i=0;i<m_Anims.size();i++) { 
       for(int j=0;j<m_Anims[i]->m_Frames.size();j++) 
@@ -858,6 +1078,11 @@ namespace vapp {
     for(int i=0;i<m_Particles.size();i++)
       delete m_Particles[i];
   }
+  
+  void GameRenderer::shutdown(void) {
+    /* Free overlay */
+    m_Overlay.cleanUp();
+  }  
 
   /*===========================================================================
   Debug info. Note how this is leaked into the void and nobody cares :) 
@@ -930,5 +1155,136 @@ namespace vapp {
     }
   }
   
-};
+  /*===========================================================================
+  Replay stuff
+  ===========================================================================*/
+  void GameRenderer::showReplayHelp(float p_speed, bool bAllowRewind) {
+    if(bAllowRewind) {
+      if(p_speed >= 10.0) {
+	m_pReplayHelp->setCaption(GAMETEXT_REPLAYHELPTEXT(String(">> 10")));
+      } else if(p_speed <= -10.0) {
+	m_pReplayHelp->setCaption(GAMETEXT_REPLAYHELPTEXT(String("<<-10")));
+      } else {
+	char v_speed_str[5 + 1];
+	sprintf(v_speed_str, "% .2f", p_speed);
+	m_pReplayHelp->setCaption(GAMETEXT_REPLAYHELPTEXT(String(v_speed_str)));
+      }
+    } else {
+      if(p_speed >= 10.0) {
+	m_pReplayHelp->setCaption(GAMETEXT_REPLAYHELPTEXTNOREWIND(String(">> 10")));
+      } else if(p_speed <= -10.0) {
+	m_pReplayHelp->setCaption(GAMETEXT_REPLAYHELPTEXTNOREWIND(String("<<-10")));
+      } else {
+	char v_speed_str[5 + 1];
+	sprintf(v_speed_str, "% .2f", p_speed);
+	m_pReplayHelp->setCaption(GAMETEXT_REPLAYHELPTEXTNOREWIND(String(v_speed_str)));
+      }
+    }
+  }
 
+  void GameRenderer::hideReplayHelp() {
+    m_pReplayHelp->setCaption("");
+  }
+  
+  /*===========================================================================
+  In-game text rendering
+  ===========================================================================*/
+  /* 
+                     |pi0
+                     |pi1
+                     |pi2
+                     |pi3
+       -------------------
+       m0 m4 m8  m12 |po0
+       m1 m5 m9  m13 |po1
+       m2 m6 m10 m14 |po2
+       m3 m7 m11 m15 |po3      
+  */
+  #define MULT_GL_MATRIX(pi,po,m) { \
+    po[0] = m[0]*pi[0] + m[4]*pi[1] + m[8]*pi[2] + m[12]*pi[3]; \
+    po[1] = m[1]*pi[0] + m[5]*pi[1] + m[9]*pi[2] + m[13]*pi[3]; \
+    po[2] = m[2]*pi[0] + m[6]*pi[1] + m[10]*pi[2] + m[14]*pi[3]; \
+    po[3] = m[3]*pi[0] + m[7]*pi[1] + m[11]*pi[2] + m[15]*pi[3]; \
+  }    
+  
+  void GameRenderer::_RenderInGameText(Vector2f P,const std::string &Text,Color c) {
+    /* Perform manual transformation of world coordinates into screen
+       coordinates */
+    GLfloat fPoint[4],fTemp[4];
+    GLfloat fModelView[16];
+    GLfloat fProj[16];
+    
+    glGetFloatv(GL_MODELVIEW_MATRIX,fModelView);    
+    glGetFloatv(GL_PROJECTION_MATRIX,fProj);
+    
+    fPoint[0] = P.x;
+    fPoint[1] = P.y;
+    fPoint[2] = 0.0f; /* no z please */
+    fPoint[3] = 1.0f; /* homogenous 4-D coords */
+    
+    MULT_GL_MATRIX(fPoint,fTemp,fModelView);
+    MULT_GL_MATRIX(fTemp,fPoint,fProj);
+    
+    float x = (fPoint[0] / fPoint[3] + 1.0f) / 2.0f;
+    float y = 1.0f - (fPoint[1] / fPoint[3] + 1.0f) / 2.0f;
+    
+    if(x > 0.0f && x < 1.0f && y > 0.0f && y < 1.0f) {    
+      /* Map to viewport */
+      float vx = ((float)getParent()->getDispWidth() * x);
+      float vy = ((float)getParent()->getDispHeight() * y);
+
+      glMatrixMode(GL_PROJECTION);
+      glPushMatrix();
+      glLoadIdentity();
+      glOrtho(0,getParent()->getDispWidth(),0,getParent()->getDispHeight(),-1,1);
+      glMatrixMode(GL_MODELVIEW);
+      glPushMatrix();
+      glLoadIdentity();
+      
+      int nMinX,nMinY,nMaxX,nMaxY;
+      UITextDraw::getTextExt(m_pSFont,Text,&nMinX,&nMinY,&nMaxX,&nMaxY);
+      int nx = vx - (nMaxX - nMinX)/2;
+      int ny = vy;
+      UITextDraw::printRaw(m_pSFont,nx-1,ny-1,Text,MAKE_COLOR(0,0,0,GET_ALPHA(c)));
+      UITextDraw::printRaw(m_pSFont,nx+1,ny-1,Text,MAKE_COLOR(0,0,0,GET_ALPHA(c)));
+      UITextDraw::printRaw(m_pSFont,nx+1,ny+1,Text,MAKE_COLOR(0,0,0,GET_ALPHA(c)));
+      UITextDraw::printRaw(m_pSFont,nx-1,ny+1,Text,MAKE_COLOR(0,0,0,GET_ALPHA(c)));
+      UITextDraw::printRaw(m_pSFont,nx,ny,Text,c);
+
+      glPopMatrix();
+      glMatrixMode(GL_PROJECTION);
+      glPopMatrix();
+      glMatrixMode(GL_MODELVIEW);
+      
+    }
+  }
+  
+  void GameRenderer::showMsgNewPersonalHighscore(String p_save) {
+    m_pInGameNewHighscore->showWindow(true);
+    m_pNewHighscorePersonal_str->showWindow(true);
+    m_pNewHighscoreBest_str->showWindow(false);
+    if(p_save == "") {
+      m_pNewHighscoreSave_str->showWindow(false);
+    } else {
+      m_pNewHighscoreSave_str->showWindow(true);
+      m_pNewHighscoreSave_str->setCaption("(Saved as " + p_save + ")");
+    }
+  }
+  
+  void GameRenderer::showMsgNewBestHighscore(String p_save) {
+    m_pInGameNewHighscore->showWindow(true);
+    m_pNewHighscorePersonal_str->showWindow(false);
+    m_pNewHighscoreBest_str->showWindow(true);
+    if(p_save == "") {
+      m_pNewHighscoreSave_str->showWindow(false);
+    } else {
+      m_pNewHighscoreSave_str->showWindow(true);
+      m_pNewHighscoreSave_str->setCaption("(Saved as " + p_save + ")");
+    }
+  }
+
+  void GameRenderer::hideMsgNewHighscore() {
+    m_pInGameNewHighscore->showWindow(false);
+  }
+
+};
