@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -26,6 +26,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VApp.h"
 #include "MotoGame.h"
 #include "GUI.h"
+#include "Replay.h"
+
+#define ZOOM_DEFAULT 0.195f
 
 namespace vapp {
 
@@ -148,16 +151,80 @@ namespace vapp {
     Vector2f Min,Max; /* AABB */
   };
 
+  struct TextureTheme {
+    /* rider */
+    Texture *RiderTorso;
+    Texture *RiderUpperLeg;
+    Texture *RiderLowerLeg;
+    Texture *RiderUpperArm;
+    Texture *RiderLowerArm;
+
+    /* bike */
+    Texture *BikeBody;
+    Texture *BikeRear;
+    Texture *BikeFront;
+    Texture *BikeWheel;
+  };
+
+	/*===========================================================================
+	Special effects overlay
+  ===========================================================================*/
+  class SFXOverlay {
+    public:
+      SFXOverlay() {
+        m_pApp = NULL;
+        m_bUseShaders = false;
+        m_VertShaderID = m_FragShaderID = m_ProgramID = 0;
+        m_DynamicTextureID = m_FrameBufferID = 0;
+        m_nOverlayWidth = m_nOverlayHeight = 0;
+      }
+    
+      /* Methods */
+      void init(App *pApp,int nWidth,int nHeight);
+      void cleanUp(void);
+      void beginRendering(void);
+      GLuint endRendering(void);
+      void fade(float f);
+      void present(void);
+    
+    private:
+      /* Some OpenGL handles */
+      GLuint m_DynamicTextureID;
+      GLuint m_FrameBufferID;      
+      int m_nOverlayWidth,m_nOverlayHeight;
+      App *m_pApp;
+      
+      /* For shaders */
+      bool m_bUseShaders;
+      GLhandleARB m_VertShaderID;
+      GLhandleARB m_FragShaderID;
+      GLhandleARB m_ProgramID;
+      
+      /* Helpers */
+      char **_LoadShaderSource(const std::string &File,int *pnNumLines);
+      void _FreeShaderSource(char **ppc,int nNumLines);
+      bool _SetShaderSource(GLhandleARB ShaderID,const std::string &File);
+  };
+
 	/*===========================================================================
 	Game rendering class
   ===========================================================================*/
   class GameRenderer {
     public:
-      GameRenderer() {m_nNumSpriteTypes=0; m_bDebug=false; m_Quality=GQ_HIGH; m_fSpeedMultiply=1.0f;}
+      GameRenderer() {
+	m_nNumSpriteTypes=0;
+	m_bDebug=false;
+	m_Quality=GQ_HIGH;
+	m_fSpeedMultiply=1.0f;
+	m_fScale = ZOOM_DEFAULT;
+	m_bGhostMotionBlur = true;
+      }
       ~GameRenderer() {_Free();}
     
       /* Methods */
       void init(void); /* only called at start-up, and not per-level */
+      void shutdown(void);
+      
       void render(void);
       void renderMiniMap(int x,int y,int nWidth,int nHeight);
       void prepareForNewLevel(void);
@@ -178,12 +245,33 @@ namespace vapp {
       UIFont *getSmallFont(void) {return m_pSFont;}
       UIFont *getMediumFont(void) {return m_pMFont;}
       void setBestTime(std::string s) {m_pBestTime->setCaption(s);}
+      void showReplayHelp(float p_speed, bool bAllowRewind);
+      void hideReplayHelp();
+#if defined(SUPPORT_WEBACCESS)  
+      void setWorldRecordTime(const std::string &s) {m_pWorldRecordTime->setCaption(s);}
+#endif
+      void setSpeed(const std::string &s) {m_pSpeed->setCaption(s);}
       std::string getBestTime(void) {return m_pBestTime->getCaption();}
       void setQuality(GraphQuality Quality) {m_Quality = Quality;}      
       void setSpeedMultiplier(float f) {m_fSpeedMultiply = f;}
-    
+      void zoom(float p_f);
+      void initZoom();
+      void setGhostMotionBlur(bool b) {m_bGhostMotionBlur = b;}
+      
+#if defined(ALLOW_GHOST)
+      void setGhostReplay(Replay *pReplay) {m_pGhostReplay = pReplay;}
+      void setGhostReplayDesc(const std::string &s) {m_ReplayDesc = s;}
+#endif
+
+      /* if p_save == "", nothing is displayed for p_save */
+      void showMsgNewPersonalHighscore(String p_save = "");
+      void showMsgNewBestHighscore(String p_save = "");
+      void hideMsgNewHighscore();
+
     private:
       /* Data */
+      float m_fScale;
+
       std::vector<GraphDebugInfo *> m_DebugInfo;
       
       std::vector<StaticGeom *> m_Geoms;
@@ -208,7 +296,16 @@ namespace vapp {
       UIWindow *m_pInGameStats;
       UIStatic *m_pPlayTime;   
       UIStatic *m_pBestTime;
-      
+      UIStatic *m_pReplayHelp;
+#if defined(SUPPORT_WEBACCESS)  
+      UIStatic *m_pWorldRecordTime;
+#endif
+      UIStatic *m_pSpeed;      
+      UIWindow *m_pInGameNewHighscore;
+      UIStatic *m_pNewHighscoreBest_str;
+      UIStatic *m_pNewHighscorePersonal_str;
+      UIStatic *m_pNewHighscoreSave_str;
+
       float m_fSpeedMultiply;
       
       UIFont *m_pMFont,*m_pSFont;
@@ -221,16 +318,19 @@ namespace vapp {
       Animation *m_pStrawberryAnim;
       Animation *m_pFlowerAnim;
       Animation *m_pWreckerAnim;
-      Texture *m_pBikeBody1;
-      Texture *m_pBikeRear1;
-      Texture *m_pBikeFront1;
-      Texture *m_pBikeWheel1;
-      Texture *m_pRiderTorso1;
-      Texture *m_pRiderUpperLeg1;
-      Texture *m_pRiderLowerLeg1;
-      Texture *m_pRiderUpperArm1;
-      Texture *m_pRiderLowerArm1;
-      
+
+      TextureTheme theme_normal;
+
+#if defined(ALLOW_GHOST)
+      TextureTheme theme_ghost;
+      Vector2f m_GhostInfoPos,m_GhostInfoVel;
+      float m_fNextGhostInfoUpdate;
+      int m_nGhostInfoTrans;
+      std::string m_GhostInfoString;
+      Replay *m_pGhostReplay;
+      std::string m_ReplayDesc;
+#endif      
+
       Texture *m_pArrowTexture;
       
       Texture *m_pSmoke1;
@@ -239,6 +339,10 @@ namespace vapp {
       Texture *m_pDirt1;
       
       GraphQuality m_Quality;
+      bool m_bGhostMotionBlur;
+      
+      /* FBO overlay */
+      SFXOverlay m_Overlay;
             
       /* Particle fun */
       float m_fNextParticleUpdate;
@@ -247,14 +351,16 @@ namespace vapp {
       /* Subroutines */
       void _RenderSprites(bool bForeground,bool bBackground);
       void _RenderSprite(Entity *pSprite);
-      void _RenderBike(void);
+      void _RenderBike(BikeState *pBike, BikeParams *pBikeParms, TextureTheme *p_theme);
       void _RenderBlocks(void);
       void _RenderBackground(void);
       void _RenderSky(void);
       void _RenderEntities(void);    
       void _RenderGameMessages(void); 
+      void _RenderGameStatus(void);
       void _RenderParticles(bool bFront=true);
       void _RenderParticle(Vector2f P,Texture *pTexture,float fSize,float fAngle,Color c);
+      void _RenderInGameText(Vector2f P,const std::string &Text,Color c = 0xffffffff);
       
       /* Helpers... */
       void _Vertex(Vector2f P);     /* Spit out a correctly transformed 
