@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -32,10 +32,8 @@ namespace vapp {
   /*===========================================================================
   Rendering of the bike
   ===========================================================================*/
-  void GameRenderer::_RenderBike(void) {
+  void GameRenderer::_RenderBike(BikeState *pBike, BikeParams *pBikeParms, TextureTheme *p_theme) {
     /* Render bike */
-    BikeState *pBike = getGameObject()->getBikeState();
-    
     Vector2f p0,p1,p2,p3,o0,o1,o2,o3;
     Vector2f C;
     Vector2f Sv,Rc,Fc;
@@ -43,16 +41,16 @@ namespace vapp {
     /* Draw front wheel */
     /* Ugly mode? */
     if(m_bUglyMode) {
-      o0 = Vector2f(-pBike->pParams->WR,0);
-      o1 = Vector2f(0,pBike->pParams->WR);
-      o2 = Vector2f(pBike->pParams->WR,0);
-      o3 = Vector2f(0,-pBike->pParams->WR);
+      o0 = Vector2f(-pBikeParms->WR,0);
+      o1 = Vector2f(0,pBikeParms->WR);
+      o2 = Vector2f(pBikeParms->WR,0);
+      o3 = Vector2f(0,-pBikeParms->WR);
     }
     else {
-      o0 = Vector2f(-pBike->pParams->WR,pBike->pParams->WR);
-      o1 = Vector2f(pBike->pParams->WR,pBike->pParams->WR);
-      o2 = Vector2f(pBike->pParams->WR,-pBike->pParams->WR);
-      o3 = Vector2f(-pBike->pParams->WR,-pBike->pParams->WR);
+      o0 = Vector2f(-pBikeParms->WR,pBikeParms->WR);
+      o1 = Vector2f(pBikeParms->WR,pBikeParms->WR);
+      o2 = Vector2f(pBikeParms->WR,-pBikeParms->WR);
+      o3 = Vector2f(-pBikeParms->WR,-pBikeParms->WR);
     }
     p0 = Vector2f(o0.x*pBike->fFrontWheelRot[0] + o0.y*pBike->fFrontWheelRot[1],
                   o0.x*pBike->fFrontWheelRot[2] + o0.y*pBike->fFrontWheelRot[3]);
@@ -83,14 +81,14 @@ namespace vapp {
       glColor3f(1,0,0);
       for(int i=0;i<nSteps;i++) {
         float r = (3.14159f * 2.0f * (float)i)/ (float)nSteps;            
-        _Vertex( Vector2f(C.x + pBike->pParams->WR*sin(r),C.y + pBike->pParams->WR*cos(r)) );
+        _Vertex( Vector2f(C.x + pBikeParms->WR*sin(r),C.y + pBikeParms->WR*cos(r)) );
       }      
       glEnd();
     }
     else {
       glEnable(GL_BLEND); 
       glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-      glBindTexture(GL_TEXTURE_2D,m_pBikeWheel1->nID);
+      glBindTexture(GL_TEXTURE_2D,p_theme->BikeWheel->nID);
       glEnable(GL_TEXTURE_2D);
       glBegin(GL_POLYGON);
       glColor3f(1,1,1);
@@ -110,16 +108,16 @@ namespace vapp {
     /* Draw rear wheel */        
     /* Ugly mode? */
     if(m_bUglyMode) {
-      o0 = Vector2f(-pBike->pParams->WR,0);
-      o1 = Vector2f(0,pBike->pParams->WR);
-      o2 = Vector2f(pBike->pParams->WR,0);
-      o3 = Vector2f(0,-pBike->pParams->WR);
+      o0 = Vector2f(-pBikeParms->WR,0);
+      o1 = Vector2f(0,pBikeParms->WR);
+      o2 = Vector2f(pBikeParms->WR,0);
+      o3 = Vector2f(0,-pBikeParms->WR);
     }
     else {
-      o0 = Vector2f(-pBike->pParams->WR,pBike->pParams->WR);
-      o1 = Vector2f(pBike->pParams->WR,pBike->pParams->WR);
-      o2 = Vector2f(pBike->pParams->WR,-pBike->pParams->WR);
-      o3 = Vector2f(-pBike->pParams->WR,-pBike->pParams->WR);
+      o0 = Vector2f(-pBikeParms->WR,pBikeParms->WR);
+      o1 = Vector2f(pBikeParms->WR,pBikeParms->WR);
+      o2 = Vector2f(pBikeParms->WR,-pBikeParms->WR);
+      o3 = Vector2f(-pBikeParms->WR,-pBikeParms->WR);
     }
     p0 = Vector2f(o0.x*pBike->fRearWheelRot[0] + o0.y*pBike->fRearWheelRot[1],
                   o0.x*pBike->fRearWheelRot[2] + o0.y*pBike->fRearWheelRot[3]);
@@ -150,14 +148,14 @@ namespace vapp {
       glColor3f(1,0,0);
       for(int i=0;i<nSteps;i++) {
         float r = (3.14159f * 2.0f * (float)i)/ (float)nSteps;            
-        _Vertex( Vector2f(C.x + pBike->pParams->WR*sin(r),C.y + pBike->pParams->WR*cos(r)) );
+        _Vertex( Vector2f(C.x + pBikeParms->WR*sin(r),C.y + pBikeParms->WR*cos(r)) );
       }      
       glEnd();
     }
     else {
       glEnable(GL_BLEND); 
       glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-      glBindTexture(GL_TEXTURE_2D,m_pBikeWheel1->nID);
+      glBindTexture(GL_TEXTURE_2D,p_theme->BikeWheel->nID);
       glEnable(GL_TEXTURE_2D);
       glBegin(GL_POLYGON);
       glColor3f(1,1,1);
@@ -195,7 +193,7 @@ namespace vapp {
 
       glEnable(GL_BLEND); 
       glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-      glBindTexture(GL_TEXTURE_2D,m_pBikeRear1->nID);
+      glBindTexture(GL_TEXTURE_2D,p_theme->BikeRear->nID);
       glEnable(GL_TEXTURE_2D);
       glBegin(GL_POLYGON);
       glColor3f(1,1,1);
@@ -231,7 +229,7 @@ namespace vapp {
       
       glEnable(GL_BLEND); 
       glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-      glBindTexture(GL_TEXTURE_2D,m_pBikeFront1->nID);
+      glBindTexture(GL_TEXTURE_2D,p_theme->BikeFront->nID);
       glEnable(GL_TEXTURE_2D);
       glBegin(GL_POLYGON);
       glColor3f(1,1,1);
@@ -265,7 +263,7 @@ namespace vapp {
       
       glEnable(GL_BLEND); 
       glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-      glBindTexture(GL_TEXTURE_2D,m_pBikeBody1->nID);
+      glBindTexture(GL_TEXTURE_2D,p_theme->BikeBody->nID);
       glEnable(GL_TEXTURE_2D);
       glBegin(GL_POLYGON);
       glColor3f(1,1,1);
@@ -312,8 +310,8 @@ namespace vapp {
         glColor3f(0,1,0);
         for(int i=0;i<nSteps;i++) {
           float r = (3.14159f * 2.0f * (float)i)/ (float)nSteps;            
-          _Vertex( Vector2f(pBike->HeadP.x + pBike->pParams->fHeadSize*sin(r),
-                            pBike->HeadP.y + pBike->pParams->fHeadSize*cos(r)) );
+          _Vertex( Vector2f(pBike->HeadP.x + pBikeParms->fHeadSize*sin(r),
+                            pBike->HeadP.y + pBikeParms->fHeadSize*cos(r)) );
         }      
         glEnd();
       }
@@ -328,7 +326,7 @@ namespace vapp {
 
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderTorso1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderTorso->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -354,7 +352,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderUpperArm1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderUpperArm->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -380,7 +378,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderLowerArm1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderLowerArm->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -406,7 +404,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderUpperLeg1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderUpperLeg->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -432,7 +430,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderLowerLeg1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderLowerLeg->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -466,8 +464,8 @@ namespace vapp {
         glColor3f(0,1,0);
         for(int i=0;i<nSteps;i++) {
           float r = (3.14159f * 2.0f * (float)i)/ (float)nSteps;            
-          _Vertex( Vector2f(pBike->Head2P.x + pBike->pParams->fHeadSize*sin(r),
-                            pBike->Head2P.y + pBike->pParams->fHeadSize*cos(r)) );
+          _Vertex( Vector2f(pBike->Head2P.x + pBikeParms->fHeadSize*sin(r),
+                            pBike->Head2P.y + pBikeParms->fHeadSize*cos(r)) );
         }      
         glEnd();
       }
@@ -482,7 +480,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderTorso1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderTorso->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -508,7 +506,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderUpperArm1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderUpperArm->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -534,7 +532,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderLowerArm1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderLowerArm->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -560,7 +558,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderUpperLeg1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderUpperLeg->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
@@ -586,7 +584,7 @@ namespace vapp {
         
         glEnable(GL_BLEND); 
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-        glBindTexture(GL_TEXTURE_2D,m_pRiderLowerLeg1->nID);
+        glBindTexture(GL_TEXTURE_2D,p_theme->RiderLowerLeg->nID);
         glEnable(GL_TEXTURE_2D);
         glBegin(GL_POLYGON);
         glColor3f(1,1,1);
