@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -22,7 +22,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 /* 
  *  Replay recording and management.
  */
-
 #include "Replay.h"
 #if defined(WIN32) 
   #include "zlib.h"
@@ -30,15 +29,23 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   #include <zlib.h>
 #endif
 
+#include <time.h> 
+#include <ctime>
+#include <iostream>
+#include <locale>
+
 namespace vapp {
 
   bool Replay::m_bEnableCompression = true;
 
   Replay::Replay() {
     m_bFinished = false;
+    m_bEndOfFile = false;
     m_fFinishTime = 0.0f;
     m_pcInputEventsData = NULL;
     m_nInputEventsDataSize = 0;
+    m_speed_factor = 1.0;
+    m_is_paused = false;
   }
       
   Replay::~Replay() {
@@ -46,6 +53,11 @@ namespace vapp {
   }        
   
   void Replay::_FreeReplay(void) {
+    /* Get rid of replay events */
+    for(int i=0;i<m_ReplayEvents.size();i++)
+      delete m_ReplayEvents[i];
+    m_ReplayEvents.clear();
+
     /* Dealloc chunks */
     for(int i=0;i<m_Chunks.size();i++) {
       if(m_Chunks[i].pcChunkData != NULL) {
@@ -170,8 +182,12 @@ namespace vapp {
       /* Try adding a .rpl extension */
       pfh = FS::openIFile(std::string("Replays/") + FileName + std::string(".rpl"));
       if(pfh == NULL) {    
-        Log("** Warning ** : Failed to open replay file for input: %s",(std::string("Replays/") + FileName).c_str());
-        return "";
+        /* Finally try opening as if it is a full path */
+        pfh = FS::openIFile(FileName);
+        if(pfh == NULL) {
+          Log("** Warning ** : Failed to open replay file for input: %s",(std::string("Replays/") + FileName).c_str());
+          return "";
+        }
       }
     }
     
@@ -293,12 +309,15 @@ namespace vapp {
     FS::closeFile(pfh);
     
     m_nCurChunk = 0;
-    m_nCurState = 0;
+    m_nCurState = 0.0;
     
+            
+    /* Reconstruct game events that are going to happen during the replay */
+    MotoGame::unserializeGameEvents(this, &m_ReplayEvents);
+
     return m_LevelID;
   }
       
-  #define STATES_PER_CHUNK 512
   void Replay::storeState(const char *pcState) {
     if(m_Chunks.empty()) {
       ReplayStateChunk Chunk;
@@ -323,132 +342,249 @@ namespace vapp {
     }
   }
   
-  bool Replay::loadState(char *pcState) {
-    /* Read next state */
-    if(m_nCurState >= STATES_PER_CHUNK) {
-      m_nCurChunk++;
-      m_nCurState = 0;
-    }
-    if(m_nCurChunk >= m_Chunks.size()) return false;    
-    if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) return false;
-    
-    memcpy(pcState,&m_Chunks[m_nCurChunk].pcChunkData[m_nCurState*m_nStateSize],m_nStateSize);
-    m_nCurState++;
-    return true;
+  bool Replay::nextNormalState() {
+    return nextState(m_speed_factor);
   }
-  
-  bool Replay::peekState(char *pcState) {
-    /* Like loadState() but this one does not advance the cursor... it just takes a peek */
-    int nPeekChunk=m_nCurChunk,nPeekState=m_nCurState;
-    if(nPeekState >= STATES_PER_CHUNK) {
-      nPeekChunk++;
-      nPeekState = 0;
+
+  bool Replay::nextState(float p_frames) {
+    if(m_is_paused) {
+      return ! m_bEndOfFile;
     }
-    if(nPeekChunk >= m_Chunks.size()) return false;
-    if(nPeekState >= m_Chunks[nPeekChunk].nNumStates) return false;
-    
-    memcpy(pcState,&m_Chunks[nPeekChunk].pcChunkData[nPeekState*m_nStateSize],m_nStateSize);
-    return true;
-  }
-  
-  std::vector<ReplayInfo *> Replay::createReplayList(const std::string &PlayerName,const std::string &LevelIDCheck) {
-    std::vector<ReplayInfo *> Ret;
-    
-    /* Find all replays done by the given player name */
-    std::vector<std::string> ReplayFiles = FS::findPhysFiles("Replays/*.rpl");
-    for(int i=0;i<ReplayFiles.size();i++) {
-      /* Try opening it */
-      FileHandle *pfh = FS::openIFile(ReplayFiles[i]);
-      if(pfh != NULL) {
-        int nVersion = FS::readByte(pfh);
-        if(nVersion == 0 || nVersion == 1) {
-          if(FS::readInt(pfh) == 0x12345678) {                  
-            std::string LevelID = FS::readString(pfh);
-            std::string Player = FS::readString(pfh);
-            float fFrameRate = FS::readFloat(pfh);
-            int nStateSize = FS::readInt(pfh);
-            bool bFinished = FS::readBool(pfh);
-            float fFinishTime = FS::readFloat(pfh);
-            
-            if((PlayerName=="" || PlayerName==Player) && FS::getFileBaseName(ReplayFiles[i]) != "Latest") {
-              if(LevelIDCheck=="" || LevelID==LevelIDCheck) {
-                /* Fine. */
-                ReplayInfo *pInfo = new ReplayInfo;
-                pInfo->Level = LevelID;
-                pInfo->Name = FS::getFileBaseName(ReplayFiles[i]);
-                pInfo->Player = Player;
-                pInfo->fFrameRate = fFrameRate;
-                
-                if(bFinished)
-                  pInfo->fFinishTime = fFinishTime;
-                else
-                  pInfo->fFinishTime = -1;
-                
-                Ret.push_back(pInfo);
-              }
-            }
-          }
-        }
-        else {
-          /* Not supported */
-        }
+
+    /* put endoffile to false ; and in the following line, put it to true if required*/
+    m_bEndOfFile = false;
+
+    m_nCurState += p_frames;
+
+    /* end or start of a chunk */
+    if(m_nCurState >= STATES_PER_CHUNK || m_nCurState < 0.0) {
+
+      /* go on the following chunk */
+      if(m_nCurState >= STATES_PER_CHUNK) { /* end of a chunk */
+	if(m_nCurChunk < m_Chunks.size() -1) {	
+	  m_nCurChunk++;
+	  m_nCurState = m_nCurState - m_Chunks[m_nCurChunk-1].nNumStates -1;
+	}
+      } else {
+	if(m_nCurState < 0.0) { /* start of a chunk */
+	  if(m_nCurChunk > 0) {	
+	    m_nCurChunk--;
+	    m_nCurState = m_Chunks[m_nCurChunk-1].nNumStates -1 + m_nCurState;
+	  }
+	}
+      }
+
+      /* if that's the beginning */
+      if(m_nCurState < 0.0) {
+	m_nCurState = 0.0;
+      }
+
+      /* if that's end */
+      if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) {
+	m_bEndOfFile = true;
+	m_nCurState = m_Chunks[m_nCurChunk].nNumStates -1;
+	return false;
+      }
       
-        FS::closeFile(pfh);
+    } else { /* that's not the end or the start of the chunk */
+
+      /* if that's the beginning, do nothing */
+
+      /* if that's end */
+      if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) {
+	m_bEndOfFile = true;
+	m_nCurState = m_Chunks[m_nCurChunk].nNumStates -1;
+	return false;
       }
     }
     
-    /* Super. */
-    return Ret;
+    return true;
+  }
+
+  void Replay::loadState(char *pcState) {
+    if(nextNormalState()) { /* do nothing */ }
+    peekState(pcState);
   }
   
-  void Replay::freeReplayList(std::vector<ReplayInfo *> &List) {
-    /* Free list items */
-    for(int i=0;i<List.size();i++)
-      delete List[i];
-    List.clear();
+  void Replay::peekState(char *pcState) {
+    /* Like loadState() but this one does not advance the cursor... it just takes a peek */
+    memcpy(pcState,
+	   &m_Chunks[m_nCurChunk].pcChunkData[((int)m_nCurState)*m_nStateSize],
+	   m_nStateSize);
   }
+
+  //std::vector<ReplayInfo *> Replay::createReplayList(const std::string &PlayerName,const std::string &LevelIDCheck) {
+  //  return ReplayList::findReplays(PlayerName,LevelIDCheck);
   
+    //std::vector<ReplayInfo *> Ret;
+    //
+    ///* Find all replays done by the given player name */
+    //std::vector<std::string> ReplayFiles = FS::findPhysFiles("Replays/*.rpl");
+    //for(int i=0;i<ReplayFiles.size();i++) {
+    //  /* Try opening it */
+    //  FileHandle *pfh = FS::openIFile(ReplayFiles[i]);
+    //  if(pfh != NULL) {
+    //    int nVersion = FS::readByte(pfh);
+    //    if(nVersion == 0 || nVersion == 1) {
+    //      if(FS::readInt(pfh) == 0x12345678) {                  
+    //        std::string LevelID = FS::readString(pfh);
+    //        std::string Player = FS::readString(pfh);
+    //        float fFrameRate = FS::readFloat(pfh);
+    //        int nStateSize = FS::readInt(pfh);
+    //        bool bFinished = FS::readBool(pfh);
+    //        float fFinishTime = FS::readFloat(pfh);
+    //        
+    //        if((PlayerName=="" || PlayerName==Player) && FS::getFileBaseName(ReplayFiles[i]) != "Latest") {
+    //          if(LevelIDCheck=="" || LevelID==LevelIDCheck) {
+    //            /* Fine. */
+    //            ReplayInfo *pInfo = new ReplayInfo;
+    //            pInfo->Level = LevelID;
+    //            pInfo->Name = FS::getFileBaseName(ReplayFiles[i]);
+    //            pInfo->Player = Player;
+    //            pInfo->fFrameRate = fFrameRate;
+    //            
+    //            if(bFinished)
+    //              pInfo->fFinishTime = fFinishTime;
+    //            else
+    //              pInfo->fFinishTime = -1;
+    //            
+    //            Ret.push_back(pInfo);
+    //          }
+    //        }
+    //      }
+    //    }
+    //    else {
+    //      /* Not supported */
+    //    }
+    //  
+    //    FS::closeFile(pfh);
+    //  }
+    //}
+    //
+    ///* Super. */
+    //return Ret;
+  //}
+  
+  //void Replay::freeReplayList(std::vector<ReplayInfo *> &List) {
+    ///* Free list items */
+    //for(int i=0;i<List.size();i++)
+    //  delete List[i];
+    //List.clear();
+  //}
+  
+  void Replay::pause() {
+    m_is_paused = ! m_is_paused;
+  }
+
+  void Replay::faster() {
+    if(m_is_paused == false) {
+      m_speed_factor += REPLAY_SPEED_INCREMENT;
+    }
+  }
+
+  void Replay::slower() {
+    if(m_is_paused == false) {
+      m_speed_factor -= REPLAY_SPEED_INCREMENT;
+    }
+  }
+
+  float Replay::getSpeed() const {
+    if(m_is_paused) {
+      return 0.0;
+    }
+    return m_speed_factor;
+  }
+
   void Replay::fastforward(float fSeconds) {
     /* How many states should we move forward? */
-    int nNumStates = (int)(fSeconds * m_fFrameRate);
-    
-    for(int i=0;i<nNumStates;i++) {
-      /* Move one state forward */
-      m_nCurState++;
-      if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) {
-        m_nCurChunk++;
-        if(m_nCurChunk >= m_Chunks.size()) {
-          m_nCurChunk = m_Chunks.size() - 1;
-          m_nCurState = m_Chunks[m_nCurChunk].nNumStates - 1;
-          break;
-        }
-        else {
-          m_nCurState = 0;
-        }
-      }
-    }
-  }  
+    float nNumStates = (fSeconds * m_fFrameRate);
+    nextState(nNumStates);
+  }
 
   void Replay::fastrewind(float fSeconds) {
-    /* How many states should we move backward? */
-    int nNumStates = (int)(fSeconds * m_fFrameRate);
-
-    for(int i=0;i<nNumStates;i++) {
-      /* Move one state back */
-      m_nCurState--;
-      if(m_nCurState < 0) {
-        m_nCurChunk--;
-        if(m_nCurChunk < 0) {
-          m_nCurChunk = 0;
-          m_nCurState = 0;
-          break;
-        }
-        else {
-          m_nCurState = m_Chunks[m_nCurChunk].nNumStates - 1;
-        }
-      }
-    }
+    /* How many states should we move forward? */
+    float nNumStates = (fSeconds * m_fFrameRate);
+    nextState(-nNumStates);
   }  
-  
-};
 
+  std::string Replay::giveAutomaticName() {
+    time_t date;
+    time(&date);
+    struct tm *TL = localtime(&date);
+
+    char date_str[256];
+    strftime(date_str, sizeof(date_str)-1, "%d-%m-%y %H_%M", TL);
+
+    return std::string(date_str);
+  }
+  
+  void Replay::reinitialize() {
+    m_nCurChunk = 0;
+    m_nCurState = 0.0;
+    m_bEndOfFile = false;
+    m_speed_factor = 1;
+    m_is_paused = false;
+
+    /* resetting events */
+    for(int i=0;i<m_ReplayEvents.size();i++) {
+      m_ReplayEvents[i]->bPassed = false;
+    }
+  }
+
+  std::string Replay::getLevelId() {
+    return m_LevelID;
+  }
+
+  void Replay::deleteReplay(std::string ReplayName) {
+    FS::deleteFile(std::string("Replays/") + ReplayName + std::string(".rpl"));
+  }
+
+  ReplayInfo* Replay::getReplayInfos(const std::string p_ReplayName) {
+    /* Found anything, and is the time stamp the same? If so, we don't 
+       want to check this again */
+
+      /* Try opening it */
+      FileHandle *pfh = FS::openIFile("Replays/" + p_ReplayName + ".rpl");
+      if(pfh == NULL) {
+	return NULL;
+      }
+
+      int nVersion = FS::readByte(pfh);
+      if(nVersion != 0 && nVersion != 1) {
+	FS::closeFile(pfh);
+	return NULL;
+      }
+
+      if(FS::readInt(pfh) != 0x12345678) {   
+	FS::closeFile(pfh);
+	return NULL;
+      }
+  
+      std::string LevelID = FS::readString(pfh);
+      std::string Player  = FS::readString(pfh);
+      float fFrameRate    = FS::readFloat(pfh);
+      int nStateSize      = FS::readInt(pfh);
+      bool bFinished      = FS::readBool(pfh);
+      float fFinishTime   = FS::readFloat(pfh);
+                
+      ReplayInfo *pRpl     = new ReplayInfo;
+      int nTimeStamp       = FS::getFileTimeStamp("Replays/" + p_ReplayName + ".rpl");
+
+      /* Set members */
+      pRpl->Level = LevelID;
+      pRpl->Name = p_ReplayName;
+      pRpl->Player = Player;
+      pRpl->nTimeStamp = nTimeStamp;
+      pRpl->fFrameRate = fFrameRate;
+
+      if(bFinished) {
+	pRpl->fFinishTime = fFinishTime;
+      } else {
+	pRpl->fFinishTime = -1;                
+      }
+          
+      FS::closeFile(pfh);
+      return pRpl;
+  }
+
+};
