@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -40,23 +40,27 @@ namespace vapp {
   bool g_bQuietLog = false;
   bool g_bVerbose = false;
   
-  void Log(char *pcFmt,...) {
+  void LogRaw(const char *pcMsg) {
+    if(!g_bQuietLog) {
+      #if defined(_MSC_VER) && defined(_DEBUG)
+        printf("%s\n",pcMsg); /* I like my stdout when debugging :P */
+      #else
+        if(g_bVerbose) 
+          printf("%s\n",pcMsg); /* also write to stdout */
+      #endif
+
+      FS::writeLog(pcMsg);
+    }
+  }
+  
+  void Log(const char *pcFmt,...) {
     va_list List;
     char cBuf[1024];
     va_start(List,pcFmt);
     vsprintf(cBuf,pcFmt,List);
     va_end(List);
     
-    if(!g_bQuietLog) {
-      #if defined(_MSC_VER) && defined(_DEBUG)
-        printf("%s\n",cBuf); /* I like my stdout when debugging :P */
-      #else
-        if(g_bVerbose) 
-          printf("%s\n",cBuf); /* also write to stdout */
-      #endif
-
-      FS::writeLog(cBuf);
-    }
+    LogRaw(cBuf);    
   }
 
   /*===========================================================================
@@ -218,6 +222,10 @@ namespace vapp {
       if(!isNoGraphics()) {
         /* Swap buffers */
         SDL_GL_SwapBuffers();
+        
+        /* Does app want us to delay a bit after the frame? */
+        if(m_nFrameDelay > 0)
+          SDL_Delay(m_nFrameDelay);
       }
     }
     
@@ -225,6 +233,13 @@ namespace vapp {
     _Uninit();
   }
 
+  bool App::haveMouseMoved(void) {
+    int nX,nY;
+    SDL_GetRelativeMouseState(&nX,&nY);
+    if(nX || nY) return true;
+    return false;
+  }
+
   void App::getMousePos(int *pnX,int *pnY) {
     int nX,nY;
     SDL_GetMouseState(&nX,&nY);
@@ -315,22 +330,6 @@ namespace vapp {
 	  /* 2005-10-05 ... note that we no longer ask for explicit settings... it's
 	                    better to do it per auto */
 	  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);	
-
-	  //switch(m_nDispBPP) {
-		 // case 16: 
-			//  SDL_GL_SetAttribute(SDL_GL_RED_SIZE,5);
-			//  SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,6);
-			//  SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,5);
-			//  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,16);
-			//  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);	
-			//  break;
-		 // case 32: 
-			//  SDL_GL_SetAttribute(SDL_GL_RED_SIZE,8);
-			//  SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,8);
-			//  SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,8);
-			//  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,16);
-			//  break;
-	  //}	
 	
 	  /* Create video flags */
 	  int nFlags = SDL_OPENGL;
@@ -361,8 +360,6 @@ namespace vapp {
 	  glViewport(0,0,m_nDispWidth,m_nDispHeight);
 	  glMatrixMode(GL_PROJECTION);
 	  glLoadIdentity();
-	  //gluOrtho2D(0,0,m_nDispWidth,m_nDispHeight);
-//	  glOrtho(0,0,m_nDispWidth,m_nDispHeight,-1,1);
 	  glOrtho(0,m_nDispWidth,0,m_nDispHeight,-1,1);
 	  
 	  glMatrixMode(GL_MODELVIEW);
@@ -394,12 +391,20 @@ namespace vapp {
       }
     #endif
 	  
-//          Log("GL: %s",glGetString(GL_EXTENSIONS));
 	  /* Init OpenGL extensions */
-	  if(m_bDontUseGLExtensions) 
+	  if(m_bDontUseGLExtensions) {
 	    m_bVBOSupported = false;
-	  else
+	    m_bFBOSupported = false;
+	    m_bShadersSupported = false;
+	  }
+	  else {
 	    m_bVBOSupported = isExtensionSupported("GL_ARB_vertex_buffer_object");
+	    m_bFBOSupported = isExtensionSupported("GL_EXT_framebuffer_object");
+	    
+	    m_bShadersSupported = isExtensionSupported("GL_ARB_fragment_shader") &&
+	                          isExtensionSupported("GL_ARB_vertex_shader") &&
+	                          isExtensionSupported("GL_ARB_shader_objects");
+	  }
 	  
 	  if(m_bVBOSupported) {
 		  glGenBuffersARB=(PFNGLGENBUFFERSARBPROC)SDL_GL_GetProcAddress("glGenBuffersARB");
@@ -410,10 +415,83 @@ namespace vapp {
 	    glEnableClientState( GL_VERTEX_ARRAY );		
 	    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
 		  	  
-	    Log("GL: using Vertex Buffer Objects");	  
+	    Log("GL: using ARB_vertex_buffer_object");	  
 	  }
 	  else
-	    Log("GL: not using Vertex Buffer Objects");	  
+	    Log("GL: not using ARB_vertex_buffer_object");	  
+	    
+	  if(m_bFBOSupported) {
+		  glIsRenderbufferEXT = (PFNGLISRENDERBUFFEREXTPROC)SDL_GL_GetProcAddress("glIsRenderbufferEXT");
+		  glBindRenderbufferEXT = (PFNGLBINDRENDERBUFFEREXTPROC)SDL_GL_GetProcAddress("glBindRenderbufferEXT");
+		  glDeleteRenderbuffersEXT = (PFNGLDELETERENDERBUFFERSEXTPROC)SDL_GL_GetProcAddress("glDeleteRenderbuffersEXT");
+		  glGenRenderbuffersEXT = (PFNGLGENRENDERBUFFERSEXTPROC)SDL_GL_GetProcAddress("glGenRenderbuffersEXT");
+		  glRenderbufferStorageEXT = (PFNGLRENDERBUFFERSTORAGEEXTPROC)SDL_GL_GetProcAddress("glRenderbufferStorageEXT");
+		  glGetRenderbufferParameterivEXT = (PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC)SDL_GL_GetProcAddress("glGetRenderbufferParameterivEXT");
+		  glIsFramebufferEXT = (PFNGLISFRAMEBUFFEREXTPROC)SDL_GL_GetProcAddress("glIsFramebufferEXT");
+		  glBindFramebufferEXT = (PFNGLBINDFRAMEBUFFEREXTPROC)SDL_GL_GetProcAddress("glBindFramebufferEXT");
+		  glDeleteFramebuffersEXT = (PFNGLDELETEFRAMEBUFFERSEXTPROC)SDL_GL_GetProcAddress("glDeleteFramebuffersEXT");
+		  glGenFramebuffersEXT = (PFNGLGENFRAMEBUFFERSEXTPROC)SDL_GL_GetProcAddress("glGenFramebuffersEXT");
+		  glCheckFramebufferStatusEXT = (PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC)SDL_GL_GetProcAddress("glCheckFramebufferStatusEXT");
+		  glFramebufferTexture1DEXT = (PFNGLFRAMEBUFFERTEXTURE1DEXTPROC)SDL_GL_GetProcAddress("glFramebufferTexture1DEXT");
+		  glFramebufferTexture2DEXT = (PFNGLFRAMEBUFFERTEXTURE2DEXTPROC)SDL_GL_GetProcAddress("glFramebufferTexture2DEXT");
+		  glFramebufferTexture3DEXT = (PFNGLFRAMEBUFFERTEXTURE3DEXTPROC)SDL_GL_GetProcAddress("glFramebufferTexture3DEXT");
+		  glFramebufferRenderbufferEXT = (PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC)SDL_GL_GetProcAddress("glFramebufferRenderbufferEXT");
+		  glGetFramebufferAttachmentParameterivEXT = (PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC)SDL_GL_GetProcAddress("glGetFramebufferAttachmentParameterivEXT");
+		  glGenerateMipmapEXT = (PFNGLGENERATEMIPMAPEXTPROC)SDL_GL_GetProcAddress("glGenerateMipmapEXT");
+	    	  
+	    Log("GL: using EXT_framebuffer_object");
+	  }
+	  else
+	    Log("GL: not using EXT_framebuffer_object");
+	    
+	  if(m_bShadersSupported) {
+      glBindAttribLocationARB = (PFNGLBINDATTRIBLOCATIONARBPROC)SDL_GL_GetProcAddress("glBindAttribLocationARB");
+      glGetActiveAttribARB = (PFNGLGETACTIVEATTRIBARBPROC)SDL_GL_GetProcAddress("glGetActiveAttribARB");
+      glGetAttribLocationARB = (PFNGLGETATTRIBLOCATIONARBPROC)SDL_GL_GetProcAddress("glGetAttribLocationARB");
+      glDeleteObjectARB = (PFNGLDELETEOBJECTARBPROC)SDL_GL_GetProcAddress("glDeleteObjectARB");
+      glGetHandleARB = (PFNGLGETHANDLEARBPROC)SDL_GL_GetProcAddress("glGetHandleARB");
+      glDetachObjectARB = (PFNGLDETACHOBJECTARBPROC)SDL_GL_GetProcAddress("glDetachObjectARB");
+      glCreateShaderObjectARB = (PFNGLCREATESHADEROBJECTARBPROC)SDL_GL_GetProcAddress("glCreateShaderObjectARB");
+      glShaderSourceARB = (PFNGLSHADERSOURCEARBPROC)SDL_GL_GetProcAddress("glShaderSourceARB");
+      glCompileShaderARB = (PFNGLCOMPILESHADERARBPROC)SDL_GL_GetProcAddress("glCompileShaderARB");
+      glCreateProgramObjectARB = (PFNGLCREATEPROGRAMOBJECTARBPROC)SDL_GL_GetProcAddress("glCreateProgramObjectARB");
+      glAttachObjectARB = (PFNGLATTACHOBJECTARBPROC)SDL_GL_GetProcAddress("glAttachObjectARB");
+      glLinkProgramARB = (PFNGLLINKPROGRAMARBPROC)SDL_GL_GetProcAddress("glLinkProgramARB");
+      glUseProgramObjectARB = (PFNGLUSEPROGRAMOBJECTARBPROC)SDL_GL_GetProcAddress("glUseProgramObjectARB");
+      glValidateProgramARB = (PFNGLVALIDATEPROGRAMARBPROC)SDL_GL_GetProcAddress("glValidateProgramARB");
+      glUniform1fARB = (PFNGLUNIFORM1FARBPROC)SDL_GL_GetProcAddress("glUniform1fARB");
+      glUniform2fARB = (PFNGLUNIFORM2FARBPROC)SDL_GL_GetProcAddress("glUniform2fARB");
+      glUniform3fARB = (PFNGLUNIFORM3FARBPROC)SDL_GL_GetProcAddress("glUniform3fARB");
+      glUniform4fARB = (PFNGLUNIFORM4FARBPROC)SDL_GL_GetProcAddress("glUniform4fARB");
+      glUniform1iARB = (PFNGLUNIFORM1IARBPROC)SDL_GL_GetProcAddress("glUniform1iARB");
+      glUniform2iARB = (PFNGLUNIFORM2IARBPROC)SDL_GL_GetProcAddress("glUniform2iARB");
+      glUniform3iARB = (PFNGLUNIFORM3IARBPROC)SDL_GL_GetProcAddress("glUniform3iARB");
+      glUniform4iARB = (PFNGLUNIFORM4IARBPROC)SDL_GL_GetProcAddress("glUniform4iARB");
+      glUniform1fvARB = (PFNGLUNIFORM1FVARBPROC)SDL_GL_GetProcAddress("glUniform1fvARB");
+      glUniform2fvARB = (PFNGLUNIFORM2FVARBPROC)SDL_GL_GetProcAddress("glUniform2fvARB");
+      glUniform3fvARB = (PFNGLUNIFORM3FVARBPROC)SDL_GL_GetProcAddress("glUniform3fvARB");
+      glUniform4fvARB = (PFNGLUNIFORM4FVARBPROC)SDL_GL_GetProcAddress("glUniform4fvARB");
+      glUniform1ivARB = (PFNGLUNIFORM1IVARBPROC)SDL_GL_GetProcAddress("glUniform1ivARB");
+      glUniform2ivARB = (PFNGLUNIFORM2IVARBPROC)SDL_GL_GetProcAddress("glUniform2ivARB");
+      glUniform3ivARB = (PFNGLUNIFORM3IVARBPROC)SDL_GL_GetProcAddress("glUniform3ivARB");
+      glUniform4ivARB = (PFNGLUNIFORM4IVARBPROC)SDL_GL_GetProcAddress("glUniform4ivARB");
+      glUniformMatrix2fvARB = (PFNGLUNIFORMMATRIX2FVARBPROC)SDL_GL_GetProcAddress("glUniformMatrix2fvARB");
+      glUniformMatrix3fvARB = (PFNGLUNIFORMMATRIX3FVARBPROC)SDL_GL_GetProcAddress("glUniformMatrix3fvARB");
+      glUniformMatrix4fvARB = (PFNGLUNIFORMMATRIX4FVARBPROC)SDL_GL_GetProcAddress("glUniformMatrix4fvARB");
+      glGetObjectParameterfvARB = (PFNGLGETOBJECTPARAMETERFVARBPROC)SDL_GL_GetProcAddress("glGetObjectParameterfvARB");
+      glGetObjectParameterivARB = (PFNGLGETOBJECTPARAMETERIVARBPROC)SDL_GL_GetProcAddress("glGetObjectParameterivARB");
+      glGetInfoLogARB = (PFNGLGETINFOLOGARBPROC)SDL_GL_GetProcAddress("glGetInfoLogARB");
+      glGetAttachedObjectsARB = (PFNGLGETATTACHEDOBJECTSARBPROC)SDL_GL_GetProcAddress("glGetAttachedObjectsARB");
+      glGetUniformLocationARB = (PFNGLGETUNIFORMLOCATIONARBPROC)SDL_GL_GetProcAddress("glGetUniformLocationARB");
+      glGetActiveUniformARB = (PFNGLGETACTIVEUNIFORMARBPROC)SDL_GL_GetProcAddress("glGetActiveUniformARB");
+      glGetUniformfvARB = (PFNGLGETUNIFORMFVARBPROC)SDL_GL_GetProcAddress("glGetUniformfvARB");
+      glGetUniformivARB = (PFNGLGETUNIFORMIVARBPROC)SDL_GL_GetProcAddress("glGetUniformivARB");
+      glGetShaderSourceARB = (PFNGLGETSHADERSOURCEARBPROC)SDL_GL_GetProcAddress("glGetShaderSourceARB");    
+	  	  
+	    Log("GL: using ARB_fragment_shader/ARB_vertex_shader/ARB_shader_objects");
+	  }
+	  else
+	    Log("GL: not using ARB_fragment_shader/ARB_vertex_shader/ARB_shader_objects");
 	  
 	  /* Set background color to black */
 	  glClearColor(0.0f,0.0f,0.0f,0.0f);
@@ -437,6 +515,70 @@ namespace vapp {
     SDL_Quit();
   }
   
+  /*===========================================================================
+  Return available display modes
+  ===========================================================================*/
+  std::vector<std::string>* App::getDisplayModes(){
+    std::vector<std::string>* modes = new std::vector<std::string>;
+    SDL_Rect **sdl_modes;
+    int i, nFlags;
+
+    /* Always use the fullscreen flags to be sure to
+       always get a result (no any modes available like in windowed) */
+    nFlags = SDL_OPENGL | SDL_FULLSCREEN;
+
+    /* Get available fullscreen/hardware modes */
+    sdl_modes = SDL_ListModes(NULL, nFlags);
+
+    /* Check is there are any modes available */
+    if(sdl_modes == (SDL_Rect **)0){
+      Log("** Warning ** : No display modes available.");
+      throw Exception("getDisplayModes : No modes available.");
+    }
+
+    /* Check if or resolution is restricted */
+    if(sdl_modes == (SDL_Rect **)-1){
+      /* Should never happen */
+      //Log("All resolutions available.");
+      modes->push_back("800 X 600");
+      modes->push_back("1024 X 768");
+      modes->push_back("1280 X 1024");
+      modes->push_back("1600 X 1200");
+    } else{
+      /* Print valid modes */
+      //Log("Available Modes :");
+
+      for(i=0; sdl_modes[i]; i++){
+	char tmp[128];
+
+	/* Menus don't fit under 800x600 */
+	if(sdl_modes[i]->w < 800 || sdl_modes[i]->h < 600)
+	  continue;
+
+	snprintf(tmp, 126, "%d X %d",
+		sdl_modes[i]->w,
+		sdl_modes[i]->h);
+	tmp[127] = '\0';
+
+	/* Only single */
+	bool findDouble = false;
+	//Log("size: %d", modes->size());
+	for(int j=0; j<modes->size(); j++)
+	  if(!strcmp(tmp, (*modes)[j].c_str())){
+	    findDouble = true;
+	    break;
+	  }
+
+	if(!findDouble){
+	  modes->push_back(tmp);
+	  //Log("  %s", tmp);
+	}
+      }
+    }
+
+    return modes;
+  }
+
   /*===========================================================================
   Set/get graphics scissoring
   ===========================================================================*/
