@@ -1,6 +1,6 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005 Rasmus Neckelmann (neckelmann@gmail.com)
+Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -25,6 +25,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #ifdef WIN32
   #include <io.h>
+  #include <direct.h>
 #else
   #include <unistd.h>
   #include <sys/types.h>
@@ -125,9 +126,92 @@ namespace vapp {
   }
 
   /*===========================================================================
-  Code soup :)  -- this is largely copy/paste material from earlier project
+  Code soup :)  -- this is largely copy/paste material from an earlier project
+  
+  FutureRasmus: well, this is probably the crappiest code in the world. Period.
   ===========================================================================*/
-  std::vector<std::string> FS::findPhysFiles(std::string Files) {
+  void FS::_FindFilesRecursive(const std::string &DirX,const std::string &Wildcard,std::vector<std::string> &List) {
+    /* Windows? */
+    #ifdef WIN32
+      long fh;
+      struct _finddata_t fd;
+      
+      std::string Dir = DirX;
+      if(!DirX.empty() && DirX[DirX.length()-1] == '/') {
+        Dir = DirX.substr(0,DirX.length()-1);
+      }
+      
+      if((fh = _findfirst((Dir + std::string("/") + Wildcard).c_str(),&fd)) != -1L) {
+        do {
+					if(strcmp(fd.name,".") && strcmp(fd.name,"..")) {
+					  std::string F = Dir + std::string("/") + std::string(fd.name);
+					  bool bFound = false;
+					  for(int k = 0;k<List.size();k++) {
+					    if(getFileBaseName(List[k]) == getFileBaseName(F)) {  
+					      bFound = true;
+					      break;
+					    }
+					  }
+					  if(!bFound)
+						  List.push_back(F);
+				  }
+        } while(_findnext(fh,&fd)==0);
+        _findclose(fh);
+      }
+      
+      /* Now, recurse into sub-dirs */
+      if((fh = _findfirst( (Dir + std::string("/*")).c_str(),&fd)) != -1L) {
+        do {
+					if(strcmp(fd.name,".") && strcmp(fd.name,"..")) {
+					  std::string F = Dir + std::string("/") + std::string(fd.name);
+            if(isDir(F)) {
+              /* Recurse! */
+              _FindFilesRecursive(F,Wildcard,List);
+            }					  
+					}
+			  } while(_findnext(fh,&fd)==0);
+			  _findclose(fh);
+			}      
+	  #else
+	    std::string Dir = DirX;
+	  
+      struct dirent *dp;    
+      DIR *dirp = opendir(Dir.c_str());
+      while(dirp) {
+        if((dp = readdir(dirp)) != NULL) {
+			    std::string F = Dir + std::string(dp->d_name);
+			    if(isDir(F)) {
+			      /* Recurse... */
+            if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {			      
+              _FindFilesRecursive(F + std::string("/"),Wildcard,List);
+            }
+			    }
+			    else {			    
+            if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
+              /* Match! */
+              if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
+					      bool bFound = false;
+					      for(int k = 0;k<List.size();k++) {
+  					      if(FS::getFileBaseName(List[k]) == FS::getFileBaseName(F)) {  
+					          bFound = true;
+					          break;
+					        }
+					      }
+					      if(!bFound)
+						      List.push_back(F);
+              }							
+            }
+          }
+        }
+        else {
+          closedir(dirp);
+          break;        
+        }
+      }	      
+    #endif    
+  }
+
+  std::vector<std::string> FS::findPhysFiles(std::string Files,bool bRecurse) {
     std::vector<std::string> Result;
     std::string Wildcard;
     std::string DirToSearch,AltDirToSearch,UDirToSearch = "";
@@ -170,57 +254,39 @@ namespace vapp {
 //    printf("W=%s   D=%s  AD=%s\n",Wildcard.c_str(),DirToSearch.c_str(),AltDirToSearch.c_str());    
     /* Windows? */
     #ifdef WIN32
-      long fh;
-      struct _finddata_t fd;
-      
-      if((fh = _findfirst((DirToSearch + Wildcard).c_str(),&fd)) != -1L) {
-        do {
-					if(strcmp(fd.name,".") && strcmp(fd.name,"..")) {
-					  std::string F = DirToSearch + std::string(fd.name);
-					  bool bFound = false;
-					  for(int k = 0;k<Result.size();k++) {
-					    if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
-					      bFound = true;
-					      break;
-					    }
-					  }
-					  if(!bFound)
-						  Result.push_back(F);
-				  }
-        } while(_findnext(fh,&fd)==0);
-        _findclose(fh);
+      if(bRecurse) {
+        _FindFilesRecursive(DirToSearch,Wildcard,Result);
       }
-    #else /* Assume linux, unix... yalla yalla... */
-      /* Search directories */
-      struct dirent *dp;    
-      DIR *dirp = opendir(DirToSearch.c_str());
-      while(dirp) {
-        if((dp = readdir(dirp)) != NULL) {
-          if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
-            /* Match! */
-            if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
-					    std::string F = DirToSearch + std::string(dp->d_name);
+      else {
+        long fh;
+        struct _finddata_t fd;
+        
+        if((fh = _findfirst((DirToSearch + Wildcard).c_str(),&fd)) != -1L) {
+          do {
+					  if(strcmp(fd.name,".") && strcmp(fd.name,"..")) {
+					    std::string F = DirToSearch + std::string(fd.name);
 					    bool bFound = false;
 					    for(int k = 0;k<Result.size();k++) {
-  					    if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+					      if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
 					        bFound = true;
 					        break;
 					      }
 					    }
 					    if(!bFound)
 						    Result.push_back(F);
-            }							
-          }
-        }
-        else {
-          closedir(dirp);
-          break;        
+				    }
+          } while(_findnext(fh,&fd)==0);
+          _findclose(fh);
         }
       }
-
-      if(UDirToSearch != "") {
-        dirp = opendir(UDirToSearch.c_str());
-        DirToSearch = UDirToSearch;
+    #else /* Assume linux, unix... yalla yalla... */
+      if(bRecurse) {
+        _FindFilesRecursive(DirToSearch,Wildcard,Result);
+      }
+      else {
+        /* Search directories */
+        struct dirent *dp;    
+        DIR *dirp = opendir(DirToSearch.c_str());
         while(dirp) {
           if((dp = readdir(dirp)) != NULL) {
             if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
@@ -229,14 +295,14 @@ namespace vapp {
 					      std::string F = DirToSearch + std::string(dp->d_name);
 					      bool bFound = false;
 					      for(int k = 0;k<Result.size();k++) {
-    					    if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+  					      if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
 					          bFound = true;
 					          break;
 					        }
 					      }
 					      if(!bFound)
 						      Result.push_back(F);
-						  }
+              }							
             }
           }
           else {
@@ -245,31 +311,71 @@ namespace vapp {
           }
         }
       }
+      
+      if(UDirToSearch != "") {
+        if(bRecurse) {
+          _FindFilesRecursive(UDirToSearch,Wildcard,Result);        
+        }
+        else {
+          struct dirent *dp;    
+          DIR *dirp = opendir(UDirToSearch.c_str());
+          DirToSearch = UDirToSearch;
+          while(dirp) {
+            if((dp = readdir(dirp)) != NULL) {
+              if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
+                /* Match! */
+                if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
+					        std::string F = DirToSearch + std::string(dp->d_name);
+					        bool bFound = false;
+					        for(int k = 0;k<Result.size();k++) {
+    					      if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+					            bFound = true;
+					            break;
+					          }
+					        }
+					        if(!bFound)
+						        Result.push_back(F);
+						    }
+              }
+            }
+            else {
+              closedir(dirp);
+              break;        
+            }
+          }
+        }
+      }
 
       if(AltDirToSearch != DirToSearch) {
-        dirp = opendir(AltDirToSearch.c_str());
-        DirToSearch = AltDirToSearch;
-        while(dirp) {
-          if((dp = readdir(dirp)) != NULL) {
-            if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
-              /* Match! */
-              if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
-					      std::string F = DirToSearch + std::string(dp->d_name);
-					      bool bFound = false;
-					      for(int k = 0;k<Result.size();k++) {
-    					    if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
-					          bFound = true;
-					          break;
+        if(bRecurse) {
+          _FindFilesRecursive(AltDirToSearch,Wildcard,Result);
+        }
+        else {
+          struct dirent *dp;    
+          DIR *dirp = opendir(AltDirToSearch.c_str());
+          DirToSearch = AltDirToSearch;
+          while(dirp) {
+            if((dp = readdir(dirp)) != NULL) {
+              if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
+                /* Match! */
+                if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
+					        std::string F = DirToSearch + std::string(dp->d_name);
+					        bool bFound = false;
+					        for(int k = 0;k<Result.size();k++) {
+    					      if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+					            bFound = true;
+					            break;
+					          }
 					        }
-					      }
-					      if(!bFound)
-						      Result.push_back(F);
-						  }
+					        if(!bFound)
+						        Result.push_back(F);
+						    }
+              }
+            }
+            else {
+              closedir(dirp);
+              break;        
             }
-          }
-          else {
-            closedir(dirp);
-            break;        
           }
         }
       }
@@ -316,11 +422,24 @@ namespace vapp {
       pfh->Type = FHT_STDIO;
     }
     else {
-      /* Maybe it's in the data package then? */
-      bool bGotIt = false;
+      /* Try current working dir */
+      pfh->fp = fopen(Path.c_str(),"rb");
+      if(pfh->fp == NULL) {
+        /* No luck. Try the user-dir then */
+        pfh->fp = fopen((m_UserDir + std::string("/") + Path).c_str(),"rb");        
+        if(pfh->fp == NULL && m_bGotDataDir) {
+          /* Not there either, the data-dir is our last chance */
+          pfh->fp = fopen((m_DataDir + std::string("/") + Path).c_str(),"rb");        
+        }
+      }
+      pfh->Type = FHT_STDIO;
+    }
+    
+    if(pfh->fp == NULL) {
+      /* No luck so far, look in the data package */
       for(int i=0;i<m_nNumPackFiles;i++) {              
         if(m_PackFiles[i].Name == Path ||
-           (std::string("./") + m_PackFiles[i].Name) == Path) {
+          (std::string("./") + m_PackFiles[i].Name) == Path) {
           /* Found it, yeah. */
           pfh->fp = fopen(m_BinDataFile.c_str(),"rb");
           if(pfh->fp != NULL) {
@@ -329,25 +448,10 @@ namespace vapp {
             pfh->nSize = m_PackFiles[i].nSize;
             pfh->nOffset = ftell(pfh->fp);
 //            printf("OPENED '%s' IN PACKAGE!\n",Path.c_str());
-            bGotIt = true;
           }          
           else break;
         }
-      }
-    
-      if(!bGotIt) {
-        /* Try current working dir */
-        pfh->fp = fopen(Path.c_str(),"rb");
-        if(pfh->fp == NULL) {
-          /* No luck. Try the user-dir then */
-          pfh->fp = fopen((m_UserDir + std::string("/") + Path).c_str(),"rb");        
-          if(pfh->fp == NULL && m_bGotDataDir) {
-            /* Not there either, the data-dir is our last chance */
-            pfh->fp = fopen((m_DataDir + std::string("/") + Path).c_str(),"rb");        
-          }
-        }
-        pfh->Type = FHT_STDIO;
-      }
+      }      
     }
     
     if(pfh->fp != NULL) { 
@@ -361,6 +465,7 @@ namespace vapp {
       }
       return pfh;
     }
+    
     delete pfh;
     return NULL;
   }
@@ -668,6 +773,19 @@ namespace vapp {
     if(n<0) return FName;
     return FName.substr(0,n);
   }
+
+  /*=========================================================================== 
+  Find out how old this file thing is... (physical file system only)
+  ===========================================================================*/
+  int FS::getFileTimeStamp(const std::string &Path) {
+    struct stat S;
+
+    if(stat(Path.c_str(),&S)) { 
+      return 0;
+    }
+    
+    return (int)S.st_mtime; /* no we don't care about exact value */
+  }
   
   /*=========================================================================== 
   Is that a dir or what? - and similar stuffin'
@@ -889,22 +1007,33 @@ namespace vapp {
       }
       
       /* If there is no Replay-dir in user-dir try making that too */
-      if(!isDir(m_UserDir + std::string("/Replays"))) {
-        if(mkdir((m_UserDir + std::string("/Replays")).c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
+      if(!isDir(getReplaysDir())) {
+        if(mkdir(getReplaysDir().c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
           Log("** Warning ** : failed to create user replay directory '%s'!",(m_UserDir + std::string("/Replays")).c_str());
         }
-        if(!isDir(m_UserDir + std::string("/Replays"))) {
+        if(!isDir(getReplaysDir())) {
           /* Still no dir... */
           throw Exception("could not create user replay directory");
         }
       }
 
+      /* Make sure we got a level cache dir */
+      if(!isDir(m_UserDir + std::string("/LCache"))) {
+        if(mkdir((m_UserDir + std::string("/LCache")).c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
+          Log("** Warning ** : failed to create user LCache directory '%s'!",(m_UserDir + std::string("/LCache")).c_str());
+        }
+        if(!isDir(m_UserDir + std::string("/LCache"))) {
+          /* Still no dir... */
+          throw Exception("could not create user LCache directory");
+        }
+      }
+
       /* The same goes for the /Levels dir */
-      if(!isDir(m_UserDir + std::string("/Levels"))) {
-        if(mkdir((m_UserDir + std::string("/Levels")).c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
+      if(!isDir(getLevelsDir())) {
+        if(mkdir(getLevelsDir().c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
           Log("** Warning ** : failed to create user levels directory '%s'!",(m_UserDir + std::string("/Levels")).c_str());
         }
-        if(!isDir(m_UserDir + std::string("/Levels"))) {
+        if(!isDir(getLevelsDir())) {
           /* Still no dir... */
           throw Exception("could not create user levels directory");
         }
@@ -991,6 +1120,60 @@ namespace vapp {
     }        
   }
 
+  /*===========================================================================
+  Hmm, for some reason I have to do this. Don't ask me why.
+  ===========================================================================*/
+  int FS::mkDir(const char *pcPath) {
+    #if defined(_MSC_VER)
+      return _mkdir(pcPath);
+    #else
+      return mkdir(pcPath,S_IRUSR|S_IWUSR|S_IRWXU);
+    #endif
+  }
+
+  /*===========================================================================
+  Endian-safe I/O helpers
+  ===========================================================================*/
+  void FS::writeShort_LE(FileHandle *pfh,short v) {    
+    writeByte(pfh,v&0xff);
+    writeByte(pfh,(v&0xff00)>>8);
+  }
+  
+  void FS::writeInt_LE(FileHandle *pfh,int v) {
+    writeByte(pfh,v&0xff);
+    writeByte(pfh,(v&0xff00)>>8);
+    writeByte(pfh,(v&0xff0000)>>16);
+    writeByte(pfh,(v&0xff000000)>>24);
+  }
+  
+  void FS::writeFloat_LE(FileHandle *pfh,float v) {
+    writeInt_LE(pfh,*((int *)&v));    
+    //unsigned char *pc=(unsigned char *)&v;
+    //for(int i=0;i<4;i++)
+    //  writeByte(pfh,pc[i]);
+  }
+  
+  int FS::readShort_LE(FileHandle *pfh) {
+    unsigned char c[2];
+    readBuf(pfh,(char *)c,2);
+    return c[0]|(c[1]<<8);
+  }
+  
+  int FS::readInt_LE(FileHandle *pfh) {
+    unsigned char c[4];
+    readBuf(pfh,(char *)c,4);
+    return c[0]|(c[1]<<8)|(c[2]<<16)|(c[3]<<24);
+  }
+
+  float FS::readFloat_LE(FileHandle *pfh) {
+    float res;
+
+    unsigned char *pc=(unsigned char *)&res;
+    for(int i=0;i<4;i++)
+    pc[i]=readByte(pfh);
+
+    return res;
+  }  
   
 };
 
